#ifndef __ACHA_H__
#define __ACHA_H__
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <arpa/inet.h>
#include <signal.h>
#include <fcntl.h>
#include <wait.h>
#include <pthread.h>
#include <errno.h>
#include <fcntl.h>
#include <termios.h>
#include <openssl/md5.h>

#include <iostream>
#include <string>
#include <deque>
#include <list>
#include <vector>
#include <algorithm>

#include "lm.h"

using namespace std;

#define CONTROL_PORT		8887
#define CONTROL_PORT_END	8787
#define CM_PORT				8888
#define CM_PORT_END			8988
#define CM_RLOCK_PATH		"/tmp/ACHA"
#define CM_PASS_LEN			0					// 패스워드의 최소길이
//#define CM_PASS_LEN			6
#define SDATA				256					// 작은데이터사이즈
#define DATASIZ				1024				// 패킷용 데이터사이즈
#define REMOTE				"127.0.0.1"			// 기본 원격지 주소
#define CONFIGDIR			".config"			// 설정파일 경로
#define BACKUPDIR			".backup"			// 설정파일 경로
#define	SYNCLISTPATH		".config/SyncList"	// Synclist 경로
#define	CFLISTPATH			".config/ChangedFiles"
#define	DELIM				" |\t\n"
//#define MAXPATHLEN 		50
#define	MAX_FILENAME		256
#define TEMPFOLDER 			".temp"
#define TMTEMP	 			".temp/tm_temp_file"
#define DEFAULT_SYNC_FILE_SIZE	1000000

#define UIM_PORT                8989
#define UIM_PORT_END            8999
#define UIM_RLOCK_PATH      "/tmp/ACHA"
#define UIM_PASS_LEN            0                   // 패스워드의 최소길이

// 출력 옵션 //
#define LOG_MODE		OUTPUT_FILE|OUPUT_CONSOLE
#define TRACE			printf("%s %s() - line %d\n", __FILE__, __FUNCTION__, __LINE__)
#define debug(X, Y)		if( acha->m_bDebug ) printf("[1;36m[DEBUG] %s - %s line %d : %s = %d[00m\n", \
							__FILE__, __FUNCTION__, __LINE__, (X), (Y))

enum {
	CM_ACK_NO,			// 명령 실행 불가 응답
	CM_ACK_OK, 			// 명령 실행 완료 응답
	CM_ACK_HASH_EQUAL,	// 해쉬코드 같음. 전송 필요 없음
	CM_ACK_PRIORITY,	// 우선순위에 따른 리턴. 전송 필요없음
	CM_ACK_UNKNOWN,		// 처리 할 수 없는 명령 응답
	
	CM_LOCK,			// 락 설정 요청
	CM_UNLOCK,			// 락 해제 요청
	CM_ADD_SYNC,		// 동기화 추가 요청
	CM_DEL_SYNC,		// 동기화 삭제 요청
	CM_ADD_FILE,		// 파일 추가 요청
	CM_ADD_DIR,			// 디렉토리 추가 요청
	CM_DEL_FILE,		// 파일 삭제 요청
	CM_DEL_DIR,			// 디렉토리 삭제 요청
	CM_SEND_FILE,		// 파일 전송
	CM_UPDATE_SYNCTIME,	// 동기화 시간 갱신

	CM_ERROR,			// 함수의 비정상 종료

	TM_INFO = 1000,		// 전송파일의 정보
	TM_OK	= 1001,		// 파일 수신 완료
	TM_NO	= 1002,		// 파일 수신 실패

	CM_TEST = 9999,		// 테스트

	LEN_CONTINUE = 10000// 계속되는 파일 전송
};

enum {
	UIM_CREATE_SYNC,
	UIM_DELETE_SYNC,
	UIM_VIEW_SYNCINFO,
	UIM_ACK_NO,			// 명령 실행 불가 응답
	UIM_ACK_OK, 		// 명령 실행 완료 응답
	
	
	UIM_LOCK,			// 락 설정 요청
	UIM_UNLOCK,			// 락 해제 요청
};


// Sync List 의 동기화 상태
enum {
	FLAG_SYNC,			// SyncFlag의 일반적인 상황
	FLAG_NEWSYNC,		// local에는 생성, remote에는 아직 미생성
	FLAG_DELSYNC,		// remote의 응답이 없어서 제거 대기상태
};

// Changed List 의 파일 변경 상태
enum { 
	DIR_COPY = 1,
	DIR_FILE = 2,
	DIR_DIR = 4, 
	DIR_DELETE = 8
};

// Changed List 의 파일 및 디렉토리 변경 우선순위
enum { 
	CI_DIRADD  = 0,
	CI_FILEADD = 1,
	CI_FILEDEL = 2, 
	CI_DIRDEL  = 3
};

// 터미널 출력 색깔 지정(ANSI)
enum {
	COLOR_NONE,		COLOR_RED,		COLOR_GREEN,
	COLOR_YELLOW,	COLOR_BLUE,		COLOR_PINK,
	COLOR_SKY,		COLOR_WHITE,	COLOR_DARKRED,
	COLOR_DARKGREEN,COLOR_BROWN,	COLOR_DARKBLUE,
	COLOR_PUPLE,	COLOR_DARKSKY,	COLOR_GRAY
};

enum {
	SHOW_SCAN,
	SHOW_LAST,
	SHOW_CHANGE
};

enum {
	FLAG_ADD = 1,
	FLAG_DELETE = 2,
	FLAG_CHANGE_OPTION = 4,
	FLAG_CHANGE_FILESIZE = 8,
	FLAG_ON_ADD,
	FLAG_OFF_ADD,
	FLAG_ON_DELETE,
	FLAG_OFF_DELETE,
	FLAG_ON_CHANGE_OPTION,
	FLAG_OFF_CHANGE_OPTION,
	FLAG_ON_CHANGE_FILESIZE,
	FLAG_OFF_CHANGE_FILESIZE,
};

/******************************
 * 함수의 리턴코드를 나타낸다.
 ******************************/
enum RETURN_CODE {
	RC_OK,
	RC_NO,
	RC_ERROR,

	RC_UNKNOWN
};

typedef struct _syncListnode {
	char *syncName;			// Sync Name ( identifier )
	char *target;			// IP or Domain
	char *localPath;		// Sync Local Directory
	char *remotePath;		// Sync Remote Directory
	char *userId;			// User Id
	char *password;			// password
	int fileSize;			// file size(byte)
	int option;				// sync option (우선순위 지정시 처리되는 방식)
	int Flag;				// Sync 의 현재 상태
	time_t lastSyncTime;	// Last Syncronized Time
} syncListNode;

// 로컬 파일 정보 구조체 : 차후 list로 관리
typedef struct _scanfileinfo
{
	int type;					// 디렉토리냐 파일이냐 등등
	char name[MAX_FILENAME];	// 파일명
	time_t mod_time;			// 수정된 시간
} scanInfo;

// FM과 CM간의 통신에 사용되는 구조체로 ChangedList를 생성한다.
typedef struct _changedinfo
{
	char lPath[MAX_FILENAME];
	char rPath[MAX_FILENAME];
	int type;
} changeInfo;		

char *gen_rpath( char *, char *, char * );// 원격지 path 만들기
char *md5file( char * );		// 파일 MD5 해쉬코드
char *md5str( char * );			// 스트링 MD5 해쉬코드
char *timeFormat( time_t );		// return readable format
char *getTargetIP( char* );		// 도메인 네임으로 IP를 받음
char *getIPfromDomain( char * );// domain -> IP address
bool isIPformat( char * );		// 입력값이 IP형식인지 체크
void setcolor( int );			// 칼라 적용
int argc( char * );				// 인자의 개수를 리턴
mode_t	getPermission( char * );// 파일의 퍼미션을 읽어서 리턴 

/*=================================================
 * ACHA Base 클래스
 * ACHA 내부에서 다른 클래스들을 pointing 하기 위해
 * ================================================*/
class ACHABase
{
	public:
		ACHALog	lm;

		ACHABase(){};
		~ACHABase(){};
};

/*==============================================
 * ACHA 클래스
 * =============================================*/
class ACHA : public ACHABase
{
	private :
		struct termios m_termios;

	public :
		int  m_fdRecordLock;
		char recordLockPath[SDATA];
		char username[SDATA];
		char userpass[SDATA];

	public :
		ACHABase *m_pCommunicationManager;
		ACHABase *m_pSyncManager;
		ACHABase *m_pFileManager;
		ACHABase *m_pUIModule;

		bool	m_bDebug;			// 디버그 모드
		bool	m_bLock;			// 동기화 잠금
		bool	m_bDemonMode;		// 데몬 모드

		ACHA(void);
		~ACHA(void);

		int  init(void); 			// acha 초기화
		int  run(void);				// acha 실행
		int  cleanup(void);			// acha 종료

		bool recordLock(void);
		bool recordUnlock(void);

		void lock() 	{ m_bLock 	= true;  };
		void unlock() 	{ m_bLock 	= false; };
		void debugON() 	{ m_bDebug 	= true;  };
		void debugOFF() { m_bDebug 	= false; };

		void setPassword(void);		// 프로그램의 password를 설정
		void echoON(void);
		void echoOFF(void);

	public :
		// Console UI 디버깅을 위한 멤버
		char m_buf[DATASIZ];			// 버퍼
		char m_cmdBuffer[DATASIZ];		// 입력버퍼
		char m_md5buf[SDATA];			// md5 해쉬코드 임시저장소

		bool getCmd(void);
		int  getCmd(char *cmd);
		int  cmdParsing(void);
		
		void dispSettings(void);
};
#endif

