/*===================================//
 * cm.cpp
 * commnunication 을 위한 프로그램
 * 네트워크 관련 모든 통신을 책임진다.
 * Written by JH. Cho.
 *===================================*/

#include "acha.h"
#include "cm.h"
#include "sm.h"

//----------------------------------------------------------
// Thread Function ===================================
// 함수 : 타이머 구현 함수
// 설명 : 주기적으로 실행할 함수를 등록한다.
//----------------------------------------------------------
void *timer( void *pthis )
{
	ACHA					*acha 	= (ACHA *)pthis;
//	SyncManager				*sm 	= (SyncManager *)acha->m_pSyncManager;
	CommunicationManager 	*cm 	= (CommunicationManager *)acha->m_pCommunicationManager;

	while( 1 )
	{
		sleep( 1 );

		if( acha->m_bLock )
		{
			continue;
		}

		cm->portRefresh();
	}

	pthread_exit(NULL);
	return 0;
}

//----------------------------------------------------------
// Thread Function 
// 함수 : 클라이언트의 접속을 처리한다.
// 설명 : 서버의 기능으로 접속을 기다리며
// 		  새로운 클라이언트가 접속하면 스레드를 생성하여
// 		  명령을 처리한다.
//----------------------------------------------------------
void *listenThread( void *pthis )
{
	ACHA					*acha 	= (ACHA *)pthis;
//	SyncManager				*sm 	= (SyncManager *)acha->m_pSyncManager;
	CommunicationManager 	*cm 	= (CommunicationManager *)acha->m_pCommunicationManager;

	pthread_mutex_t	mutx;
	int status;
	cm->m_bClientConnected = false;

	if( pthread_mutex_init( &mutx, NULL ) )
	{
		cm->lm.log(LV_ERROR, "ListenThread() - 뮤텍스 생성 실패");
		return 0;
	}

	// 다음 접속을 기다린다.
	while( true )
	{
		int sock;
		sock = accept( cm->server_sockfd, 
						(struct sockaddr *)&cm->client_addr, 
						&cm->client_len );
		if( sock == -1 )
		{
			cm->lm.log(LV_ERROR, "clientConnect() Accept error");
			return 0;
		}
	
		// 접속중인 Client가 있으면 접속을 받지 않는다.
pthread_mutex_lock( &mutx );
		if( !cm->m_bClientConnected )
		{
			cm->m_bClientConnected = true;
			// 디버깅을 위한 정보를 저장
			// 추후 연결된 클라이언트 리스트로 관리
			cm->client_sockfd = sock;
			cm->m_ConnectedClientAddr = cm->client_addr;

			status = pthread_create( &cm->m_tidAcceptThread, NULL, acceptThread, pthis);
		}
		else
		{
			cm->lm.log(LV_INFO, "접속중인 클라이언트가 존재합니다");
			// sock으로 
			close(sock);
		}
pthread_mutex_unlock( &mutx );

		usleep(10);
	}

	return 0;
}

//----------------------------------------------------------
// Thread Function 
// 함수 : 클라이언트의 접속을 처리한다.
// 설명 : 서버의 기능으로 접속을 기다리며
// 		  접속된 클라이언트로부터 명령을 받아 처리한다
// 		  더이상 받을 메시지가 없으면 종료
//----------------------------------------------------------
void *acceptThread( void *pthis )
{
	ACHA					*acha 	= (ACHA *)pthis;
//	SyncManager				*sm 	= (SyncManager *)acha->m_pSyncManager;
	CommunicationManager 	*cm 	= (CommunicationManager *)acha->m_pCommunicationManager;

	int 		clientSockFd = cm->client_sockfd;
	int 		nRet;
	clientInfo 	cltmp;
	cmdData		recvCmd;		// 수신 cmd 구조체
	cmdData		respCmd;		// 응답 cmd 구조체

	// 스레드의 TID를 얻어오는 함수 찾아서 집어 넣을 것.
	cltmp.tid = cm->m_tidAcceptThread;
	cltmp.clientSockFd = clientSockFd;
	cltmp.clientIP = inet_ntoa( cm->client_addr.sin_addr );

	cm->lm.log( LV_INFO, "Client Connected from %s(%d)",
				cltmp.clientIP.c_str(), clientSockFd );

	if( cm->acha->m_bDebug )
	{
		setcolor( COLOR_YELLOW );
		cout << "\nClient Connected::sockfd = " << clientSockFd;
		cout << " (form " << cltmp.clientIP << ")" << endl;
		setcolor( COLOR_NONE );
	}

	// 접속된 Client로 부터 명령을 계속 수신한다.
	while( true )
	{
		// 명령을 읽어온다
		memset( &recvCmd, 0, sizeof(recvCmd) );
		nRet = read( clientSockFd, &recvCmd, sizeof(recvCmd.type) + sizeof(recvCmd.len) );

		// 접속이 끊어졌을 경우 
		if( nRet <= 0 )
		{
			close( clientSockFd );
			cm->m_bClientConnected = false;

			cm->lm.log( LV_INFO, "Client DisConnected from %s(%d)",
					cltmp.clientIP.c_str(), clientSockFd );

			if( cm->acha->m_bDebug )
			{
				setcolor( COLOR_YELLOW );
				cout << "\nDisConnected::sockfd = " << clientSockFd;
				cout << " (form " << cltmp.clientIP << ")" << endl;
				setcolor( COLOR_NONE );
			}
			break;
		}

		// 더 읽어올 정보가 있으면 읽어온다.
		if( recvCmd.len > 0 )
		{
			read( clientSockFd, &recvCmd.data, recvCmd.len );
		}

		if( acha->m_bDebug )
		{
			setcolor( COLOR_SKY );
			printf("[Recv] [%d] [%d] [%s]\n", recvCmd.type, recvCmd.len, recvCmd.data  );
			setcolor( COLOR_NONE );
		}

		// 읽어온 명령을 해석한다
		cm->recvDataParsing( pthis, recvCmd, respCmd );

		if( acha->m_bDebug )
		{
			setcolor( COLOR_GREEN );
			printf("[Resp] [%d] [%d] [%s]\n\n", respCmd.type, respCmd.len, respCmd.data  );
			setcolor( COLOR_NONE );
		}

		// 실행 결과 송출
		write( clientSockFd, &respCmd, sizeof(respCmd.type) + sizeof(respCmd.len) + respCmd.len );
	}
	return 0;
}

CommunicationManager::CommunicationManager(ACHA *pAcha)
{
	acha = pAcha;
}

CommunicationManager::~CommunicationManager(void)
{
}

/*
 * 함수 : 초기화 함수
 */
int CommunicationManager::init( void )
{
	lm.SetOwner("CM");
	lm.SetOutputMode(OUTPUT_FILE);
	lm.SetLogLevel(LV_INFO);

#ifdef TEST
//	lm.SetOutputMode(OUTPUT_FILE|OUTPUT_CONSOLE);
	lm.SetLogLevel(LV_TRACE);
#endif

	bzero( remoteTarget, sizeof(remoteTarget) );
	bzero( cmdBuffer, sizeof(cmdBuffer) );

	listenPort 	= CM_PORT;
	sendPort	= CM_PORT;
	controlPort	= CONTROL_PORT;

	client_len 	= sizeof(client_addr);
    server_len 	= sizeof(server_addr);
	rtarget_len = sizeof(rtarget_addr);

	memset( &serverData, 0, sizeof(serverData) );
	memset( &serverResult, 0, sizeof(serverResult) );
	memset( &clientData, 0, sizeof(clientData) );
	memset( &clientResult, 0, sizeof(clientResult) );
	memset( m_sCurSyncName, 0, sizeof(m_sCurSyncName) );

	// 디버깅을 위한 타겟 로컬지정
	strcpy( remoteTarget, "127.0.0.1" );

	init_server();
	bind_server();

	return RC_OK;
}
/*
 * 함수 : 실행
 * 설명 : 의미없음. CM의 기본 실행 루틴
 */
int CommunicationManager::run( void )
{
	int status;
    status = pthread_create( &m_tid, NULL, listenThread, acha);
    if( status < 0 ) 
    {   
        lm.log(LV_TRACE, "CM Thread 생성 실패");
    }   
    else
    {   
        lm.log(LV_TRACE, "CM Thread 생성 성공 : tid = %d", m_tid);
    }

	return RC_OK;
}

// 함수 : 명령을 분석하는 함수
// 설명 : recv 의 명령을 분석항여 해당 루틴을 처리 한 뒤
//        resp 에 결과를 넣어서 리턴
// 반환 : 처리 결과
int CommunicationManager::recvDataParsing( void *pt, cmdData& recv, cmdData& resp )
{
	ACHA					*acha 	= (ACHA *)pt;
	SyncManager				*sm 	= (SyncManager *)acha->m_pSyncManager;
	CommunicationManager 	*cm 	= (CommunicationManager *)acha->m_pCommunicationManager;

	bool 	bRet				= false;
	int  	nRet				= 0;
	int  	enumAckCode			= CM_ACK_NO;
	string	sAckCaption			= "";
	char 	szBuf[SDATA]		= "";
	char 	filename[SDATA]		= "";
	char 	hashcode[SDATA]	 	= "";

	char	szSyncName[SDATA] 		= "";
	char 	szTarget[SDATA] 		= "";
	char 	szLocalPath[SDATA] 		= "";
	char 	szRemotePath[SDATA] 	= "";

	bzero( szBuf, sizeof(szBuf) );
	bzero( filename, sizeof(filename) );
	bzero( hashcode, sizeof(hashcode) );

	switch( recv.type )
	{
		// TO DO : 각 SyncName에 따라 Lock을 따로 구성
		case CM_LOCK :		// Lock 요청
			if( acha->m_bLock )
			{
				enumAckCode = CM_ACK_NO;
				sAckCaption = "Lock Fail - Server is already locked!";
			}
			else
			{
				acha->m_bLock = true;
				sscanf( recv.data, "%s", cm->m_sCurSyncName );
				debug(cm->m_sCurSyncName, 0);

				enumAckCode = CM_ACK_OK;
				sAckCaption = "Lock OK";
			}
			break;

		// TO DO : 각 SyncName에 따라 UnLock을 따로 구성
		case CM_UNLOCK :	// Unlock 요청
			acha->m_bLock = false;
			memset( cm->m_sCurSyncName, 0, sizeof(cm->m_sCurSyncName) );

			enumAckCode = CM_ACK_OK;
			sAckCaption = "Unlock OK";
			break;

		case CM_ADD_DIR :	// 디렉토리 생성
			mode_t perm;
			sscanf( recv.data, "%s %d", filename, &perm );
			if( access( filename, F_OK|W_OK ) )
			{
				mkdir( filename, perm );
			}

			enumAckCode = CM_ACK_OK;
			sAckCaption = "Add Directory Success";
			break;

		case CM_DEL_DIR :	// 디렉토리 삭제
			sscanf( recv.data, "%s", filename );
			if( !access( filename, F_OK|W_OK ) )
			{
				rmdir( filename );
			}

			enumAckCode = CM_ACK_OK;
			sAckCaption = "Del Directory Success";
			break;

		case CM_ADD_FILE :	// 파일 생성
			sscanf( recv.data, "%s %s", filename, hashcode );
			// 파일이 존재하면 
			if( !access( filename, F_OK|W_OK ) )
			{
				// Hashcode를 비교 - 같으면 전송 안해도 된다.
				if( strcmp( md5file(filename), hashcode) == 0 )
				{
					enumAckCode = CM_ACK_HASH_EQUAL;
					sAckCaption = "Hash Code Equals - Same File";
					break;
				}

				// 옵션의 우선순위를 비교 - 높으면 전송 안해도 된다.
				if( 0 )
				{	// 내가 우선순위에 앞서는 파일을 소유중일 때
					enumAckCode = CM_ACK_PRIORITY;
					sAckCaption = "Server has high priority file";
					break;
				}
			}

			// 파일이 존재하지 않거나 위 루틴에 걸리지 않았을 경우
			enumAckCode = CM_ACK_OK;
			sAckCaption = "Receive File Ready!";

			bRet = cm->makeCmd( resp, enumAckCode, (char *)sAckCaption.c_str() );
			write( cm->client_sockfd, &resp, 
					sizeof(resp.type) + sizeof(resp.len) + resp.len );

			//--------------------------------------------
			// 파일 전송 부분. 추후 FM thread로 분리 할 것
			// CM을 이용한 파일 수신
			enumAckCode = cm->receiveFile();
			sAckCaption = "Receive File Complete";
			//--------------------------------------------
			break;

		case CM_DEL_FILE :	// 파일 삭제
			sscanf( recv.data, "%s", filename );

			if( 0 )
			{
				// 내가 우선순위에 앞서는 파일을 소유중일 때
				enumAckCode = CM_ACK_PRIORITY;
				sAckCaption = "Server has high priority file";
			}
			else
			{
				// backupFile //

				// 내가 우선순위에 밀리는 파일을 소유중일 때
				if( !access( filename, F_OK ) && remove( filename ) )
				{
					cm->lm.log( LV_ERROR, "파일 삭제 실패" );
					if( cm->acha->m_bDebug )
					{
						cout << "CM_DELFILE :: 파일 삭제 실패" << endl;
					}

					enumAckCode = CM_ACK_NO;
					sAckCaption = "Delete File Fail";
				}
				else
				{
					enumAckCode = CM_ACK_OK;
					sAckCaption = "Delete File Success";
				}
			}
			break;

		case CM_UPDATE_SYNCTIME :
			sscanf( recv.data, "%s", szBuf );
			nRet = sm->updateSyncTime( szBuf, time(NULL) );
			if( nRet != RC_OK )
			{
				enumAckCode = CM_ACK_NO;
				sAckCaption = "Update Sync Time Fail";
			}
			else
			{
				enumAckCode = CM_ACK_OK;
				sAckCaption = "Update Sync Time Success";
			}
			break;

		case CM_ADD_SYNC :			// Sync 추가
			// [sync name] [local path] [remote path] 형식 이므로
			// 수신측에서는 local path 와 remote path 를 바꾼다.
			sscanf( recv.data, "%s %s %s", szSyncName, szRemotePath, szLocalPath );
			strncpy( szTarget, inet_ntoa(cm->m_ConnectedClientAddr.sin_addr), SDATA );
			nRet = sm->addSync( szSyncName, szTarget, szLocalPath, szRemotePath, 
					FLAG_SYNC, "id", "pass", DEFAULT_SYNC_FILE_SIZE);
			if( nRet != RC_OK )
			{
				enumAckCode = CM_ACK_NO;
				sAckCaption = "Add Sync Fail";
			}
			else
			{
				enumAckCode = CM_ACK_OK;
				sAckCaption = "Add Sync Success";
			}
			break;

		case CM_DEL_SYNC :			// Sync 삭제
			sscanf( recv.data, "%s", szBuf );
			nRet = sm->delSyncNode( szBuf );
			if( nRet != RC_OK )
			{
				enumAckCode = CM_ACK_NO;
				sAckCaption = "Delete Sync Fail";
			}
			else
			{
				enumAckCode = CM_ACK_OK;
				sAckCaption = "Delete Sync Success";
			}
			break;

		case CM_TEST :			// 테스트 코드
			bRet = cm->makeCmd( resp, CM_ACK_OK, szBuf );
			break;

		default :
			bRet = cm->makeCmd( resp, CM_ACK_UNKNOWN, "Unknown Command Type!" );
			break;
	}

	// 응답 커맨드를 생성한다
	bRet = cm->makeCmd( resp, enumAckCode, (char *)sAckCaption.c_str() );

	// 결과 출력
	if( bRet ) 
	{
		return RC_OK;
	}
	else
	{
		return RC_NO;
	}
}

/*
 * 함수 : 메모리 정리 함수
 */
int CommunicationManager::cleanup( void )
{
	return RC_OK;
}

/*
 * 함수 : 컨트롤 서버를 초기화 한다.
 */
void CommunicationManager::init_control_server( void )
{
    if( (control_sockfd = socket(PF_INET, SOCK_STREAM, 0)) < 0 )
    {
        perror("init_server() socket error : ");

		return;
    }

	bzero( &control_addr, sizeof(control_addr) );
    server_addr.sin_family = AF_INET;
    server_addr.sin_addr.s_addr = htonl(INADDR_ANY);		// 자신의 IP 사용
    server_addr.sin_port = htons( controlPort );

	return;
}

/*
 * 함수 : 소켓 통신을 위한 서버 초기화
 * 설명 : 기초적인 서버의 정보를 초기화 한다.
 */
void CommunicationManager::init_server( void )
{
    if( (server_sockfd = socket(PF_INET, SOCK_STREAM, 0)) < 0 )
    {
		lm.log(LV_ERROR, "init_server() socket error : ");
		return;
    }

	bzero( &server_addr, sizeof(server_addr) );
    server_addr.sin_family = AF_INET;
    server_addr.sin_addr.s_addr = htonl(INADDR_ANY);		// 자신의 IP 사용
    server_addr.sin_port = htons( listenPort );

	return;
}

/*
 * 함수 : 상위포트로 이동
 * 설명 : 현재 listen중인 포트보다 상위포트가 비어있으면 할당
 */
void CommunicationManager::portRefresh( void )
{
	int tmp_sockfd;

	// 현재 프로그램이 최 상위포트 이면 그냥 종료
	if( listenPort == CM_PORT )
	{
		return;
	}

	// 프로그램이 최상위 포트가 아닐 경우 
	for( int i = CM_PORT; i < listenPort; i++ )
	{
		if( (tmp_sockfd = socket(PF_INET, SOCK_STREAM, 0)) < 0 )
		{
			perror("init_server() socket error : ");

			return;
		}

		bzero( &server_addr, sizeof(server_addr) );
		server_addr.sin_family = AF_INET;
		server_addr.sin_addr.s_addr = htonl(INADDR_ANY);		// 자신의 IP 사용
		server_addr.sin_port = htons( listenPort );
	}
	return;
}

/*
 * 함수 : 서버 소켓의 주소 할당
 * 설명 : 기본 포트 이후로 검색하며 비어 있는 포트에 주소를 할당
 *        할당된 포트는 멤버변수 listenPort에 저장
 */
void CommunicationManager::bind_server( void )
{
    int state = -1;

	// 포트 할당
	while( state < 0 )
    {
		server_addr.sin_port = htons( listenPort );
		state = bind( server_sockfd, (struct sockaddr *)&server_addr, server_len );

		if( state < 0 )
		{
			if( errno == EADDRINUSE )
			{
				if( acha->m_bDebug )
				{
					setcolor( COLOR_RED );
					cout << listenPort << " 포트 사용중" << endl;
					cout << "새로운 포트 할당 시도중.." << endl;
					setcolor( COLOR_NONE );
				}

				listenPort++;

				if( listenPort > CM_PORT_END )
				{
					lm.log(LV_ERROR, "bind_server() - 포트 할당 실패.");
					exit(0);
				}

				continue;
			}
	
			lm.log(LV_ERROR, "bind_server() bind error");
		}

		// 포트 할당 성공
		if( state >= 0 )
		{
			lm.log(LV_INFO, "Listen Port Binding : %d", listenPort);
			break;
		}
    }
	
    state = listen( server_sockfd, 10 );
    if( state == -1 )
    {
		lm.log(LV_ERROR, "func:bind_server() - Listen() error");
        exit(0);
    }

	return;
}

/*
 * 함수 : 소켓 통신을 위한 클라이언트 초기화
 * 인자 : 없음
 */
int CommunicationManager::init_client()
{
	return init_client( remoteTarget, sendPort );
}
int CommunicationManager::init_client( char *szTarget, int port )
{
	rtarget_addr.sin_family = AF_INET;
	rtarget_addr.sin_port = htons( port );

	SetTarget( szTarget );

	// 원격지 주소가 IP일 경우
	if( isIPformat( szTarget ) )
	{
		rtarget_addr.sin_addr.s_addr = inet_addr( remoteTarget );
	}
	// 원격지 주소가 domain name 일 경우
	else
	{
		struct hostent *he;
		he = gethostbyname( remoteTarget );

		// gethostbyname 결과 IP를 받아오지 못했을 경우
		if( he == NULL )
		{
			return RC_NO;
		}
		else
		{
			rtarget_addr.sin_addr.s_addr = inet_addr( inet_ntoa( *(struct in_addr*)he->h_addr) );
		}
	}

	return RC_OK;
}

/*
 * 함수 : 원격지 서버에 접속한다.
 * 인자 : 서버 주소, 포트
 */
int CommunicationManager::connect_client( char *szTarget, int port )
{
	int nRet;

	nRet = init_client( szTarget, port );
	if( nRet != RC_OK )
	{
		return nRet;
	}

	return connect_client();
}
int CommunicationManager::connect_client( void )
{
	int nRet;

	rtarget_sockfd = socket(PF_INET, SOCK_STREAM, 0);
    if( rtarget_sockfd < 0 )
    {
		lm.log(LV_ERROR, "connect_client() socket() failed");
		return RC_NO;
    }

	nRet = connect(rtarget_sockfd, (struct sockaddr *)&rtarget_addr, rtarget_len);
    if( nRet < 0)
    {
		lm.log(LV_INFO, "connect_client() connect() failed");
		if( rtarget_sockfd )
		{
			close( rtarget_sockfd );
		}
		return RC_NO;
    }

	return RC_OK;
}

/*
 * 함수 : clientData에 있는 내용을 원격지로 전송한다.
 * 인자 : 없음
 * 반환 : 성공 : 0, 실패 : -1
 */
int CommunicationManager::sendCmd( int type )
{
	return sendCmd( type, NULL );
}

int CommunicationManager::sendCmd( int type, char *arg )
{
	cmdData cmd;
	
	bzero( &cmd, sizeof(cmd) );
	makeCmd( cmd, type, arg );

	return sendCmd( cmd );
}

int CommunicationManager::sendCmd(cmdData cmd)
{
	int nCmdSize	= 0;
	int nRet		= 0;

	// 전송할 소켓의 유효성 검사
	if( rtarget_sockfd <= 0 )
	{
		lm.log(LV_INFO, "SendCmd() : target_sockfd is not valild");
		if( acha->m_bDebug ) 
			printf("Target_sockfd is not valid.\n");

		m_rcSend = CM_ACK_NO;
		return CM_ACK_NO;
	}

	// 커맨드 전송
	nCmdSize = sizeof(cmd.type) + sizeof(cmd.len) + cmd.len;
	nRet = write( rtarget_sockfd, &cmd, nCmdSize );
	if( nRet == nCmdSize )
	{
		lm.log( LV_DEBUG, "[Send] [%d] [%d] [%s]", cmd.type, cmd.len, cmd.data );

		if( acha->m_bDebug )
		{
			setcolor( COLOR_GREEN );
			printf("[Send] [%d] [%d] [%s]\n", cmd.type, cmd.len, cmd.data  );
			setcolor( COLOR_NONE );
		}
	}
	else
	{
		lm.log( LV_ERROR, "write() fail [CmdSize %d] - [write %d] [Send] [%d] [%d] [%s]",
				nCmdSize, nRet, cmd.type, cmd.len, cmd.data );
	}

	// 결과값 수신
	memset( &clientResult, 0, sizeof( clientData ) );
	read( rtarget_sockfd, &clientResult, 
			sizeof(clientResult.type) + sizeof(clientResult.len) );
	if( clientResult.len > 0 )
	{
		read( rtarget_sockfd, &clientResult.data, clientResult.len );
	}
	lm.log(LV_DEBUG, "[Recv] [%d] [%d] [%s]", 
				clientResult.type, clientResult.len, clientResult.data  );

	if( acha->m_bDebug )
	{
		setcolor( COLOR_SKY );
		printf("[Recv] [%d] [%d] [%s]\n\n", 
				clientResult.type, clientResult.len, clientResult.data  );
		setcolor( COLOR_NONE );
	}

	return clientResult.type;
}

/*
 * 함수 : clientData에 있는 내용을 원격지로 전송한다.
 * 인자 : 없음
 * 반환 : 성공 : 0, 실패 : -1
 */
int CommunicationManager::sendCmd( void )
{
	if( rtarget_sockfd <= 0 )
	{
		if( acha->m_bDebug ) printf("Target_sockfd is not valid.\n");
		m_rcSend = CM_ACK_NO;
		return CM_ACK_NO;
	}

	// 전송을 정해진 양만큼만 보냄
	write( rtarget_sockfd, &clientData, 
		   sizeof(clientData.type) + sizeof(clientData.len) + strlen(clientData.data) );
	lm.log(LV_DEBUG, "[Send] [%d] [%d] [%s]", 
			clientData.type, clientData.len, clientData.data );

	if( acha->m_bDebug )
	{
		setcolor( COLOR_GREEN );
		printf("[Send] [%d] [%d] [%s] -- ", 
				clientData.type, clientData.len, clientData.data  );
		setcolor( COLOR_NONE );
	}

	// 결과값 수신
	memset( &clientResult, 0, sizeof( clientData ) );
	read( rtarget_sockfd, &clientResult, 
			sizeof(clientResult.type) + sizeof(clientResult.len) );
	if( clientResult.len > 0 )
	{
		read( rtarget_sockfd, &clientResult.data, clientResult.len );
	}
	lm.log(LV_DEBUG, "[Recv] [%d] [%d] [%s]", 
				clientResult.type, clientResult.len, clientResult.data  );

	if( acha->m_bDebug )
	{
		setcolor( COLOR_SKY );
		printf("[Receive] [%d] [%d] [%s]\n", 
				clientResult.type, clientResult.len, clientResult.data  );
		setcolor( COLOR_NONE );
	}

	return clientResult.type;
}

/*
 * 함수 : buf 에 있는 내용으로 대답한다.
 * 인자 : 없음
 * 반환 : 성공 : 0, 실패 : -1
 */
int CommunicationManager::reply( void )
{
	/*
	 * 현재 구현 불가.
	 * 서버에서 전송받는 부분과 메시지를 전송하는 부분이 분리되어 있음
	 */
	write( rtarget_sockfd, &clientData, sizeof(clientData.type) + sizeof(clientData.len) );
	write( rtarget_sockfd, &clientData.data, strlen(clientData.data) );
	printf("\n[Reply Send] [%d] [%d] [%s]\n", 
			clientData.type, clientData.len, clientData.data  );

	memset( &clientResult, 0, sizeof( clientData ) );
	read( rtarget_sockfd, &clientResult, sizeof(clientResult.type) + sizeof(clientResult.len) );
	if( clientResult.len > 0 )
	{
		read( rtarget_sockfd, &clientResult.data, clientResult.len );
	}
	printf("\n[Receive] [%d] [%d] [%s]\n", 
			clientResult.type, clientResult.len, clientResult.data  );

	return 0;
}

/*
 * 함수 : 원격지와의 연결을 종료한다.
 * 인자 : 없음
 * 반환 : 성공 : 0, 실패 : 1
 */
int CommunicationManager::disconnect_client( void )
{
	close( rtarget_sockfd );
	m_bClientConnected = false;

	return 0;
}

/*
 * 함수 : 커맨드를 만들어 리턴한다.
 * 설명 : 스태틱 변수를 사용하므로 일회성 커맨드임을 명심!!
 */
bool CommunicationManager::makeCmd( cmdData &cmd, int cmdType )
{
	return makeCmd( cmd, cmdType, NULL );
}
bool CommunicationManager::makeCmd( cmdData &cmd, int cmdType, char *arg )
{
	memset( &cmd, 0, sizeof(cmd) );

	cmd.type 	= cmdType;
	if( NULL != arg )
	{
		if( strlen(arg) >= sizeof(cmd.data) )
		{
			lm.log(LV_ERROR, "makeCmd() - args가 data size를 초과하였습니다.");
			return false;
		}
		cmd.len		= strlen(arg);
		strncpy( cmd.data, arg, strlen(arg) );
	}

	return true;
}

/*
 * 함수 : 접속할 원격지의 주소를 설정한다. 
 */
void CommunicationManager::SetTarget( char *sTarget )
{
	bzero( remoteTarget, sizeof(remoteTarget) );
	strncpy( remoteTarget, sTarget, strlen(sTarget) );
	return;
}

/*
 * 함수 : 클래스 내부 변수들 확인
 */
void CommunicationManager::dispSettings( void )
{
	list<clientInfo>::iterator clIndex;

	setcolor( COLOR_SKY );
	cout << "[====================  CM Information  ====================]\n";
	setcolor( COLOR_NONE );

	cout << "[1m";
	cout << " Server_sockfd / port    : " << server_sockfd << " / " << listenPort << endl;
	cout << " LastClient IP           : " << inet_ntoa( client_addr.sin_addr ) << endl;
	cout << " RecvData                : ";
	printf( "[%d] [%d] [%s]\n", serverData.type, serverData.len, serverData.data );
	cout << "[00m";

	setcolor( COLOR_YELLOW );
	if( m_bClientConnected )
	{
		cout << " Connected Client info   : ";;
		printf( "%s(%d):%d\n", 
				inet_ntoa(m_ConnectedClientAddr.sin_addr), 
				client_sockfd,
				(int)m_ConnectedClientAddr.sin_port);
	}
	setcolor( COLOR_NONE );

	cout << "[1m";
	cout << " Target_sockfd / port    : " << rtarget_sockfd << " / " << sendPort << endl;
	cout << " Remote target IP        : " << remoteTarget << endl;
	cout << " LastRemote IP           : " << inet_ntoa( rtarget_addr.sin_addr ) << endl;
	cout << " SendData                : ";
	printf( "[%d] [%d] [%s]\n", clientData.type, clientData.len, clientData.data );
	cout << "[00m";

//	cout << " Connected Client List   : " << clientList.size() << endl;
//	for( clIndex = clientList.begin(); clIndex != clientList.end(); clIndex++ )
//	{
//		cout << "  [IP : " << clIndex->clientIP << "] [TID : ";
//		cout << clIndex->tid << "] [SOCKFD : ";
//		cout << clIndex->clientSockFd << "]" << endl;
//	}

	return;
}

/*
 * 함수 : 파일을 전송한다.
 * 인자 : lPath : 로컬의 전송대상 파일
 *        rPath : 원격지에 생성될 파일
 *        hashcode : 로컬 파일의 md5 해쉬코드
 * 반환 : CM_
 */
int CommunicationManager::sendFile( char *lPath, char *rPath, char *hashcode )
{
	int fd;
	int buffer[DATASIZ];
	struct stat status;
	mode_t filePermission;

	fd = open( lPath, O_RDONLY );
	if (fd < 0)
	{
		cout << "CM::sendFile() - Can't Open File" << endl;
		return CM_ERROR;
	}
	fstat(fd, &status);

	totalSize 		= status.st_size;
	// ## linux version
	filePermission 	= status.st_mode;
	// ## windows version
	// filePermission 	= status.st_mode;

	// 기본정보 전송
	memset( &clientData, 0, sizeof(clientData) );
	clientData.type = TM_INFO;
	sprintf( clientData.data, "%s|%d|%s|%d", rPath, totalSize, hashcode, filePermission );
	clientData.len = strlen( clientData.data );

	write( rtarget_sockfd, &clientData, 
			sizeof(clientData.type) + sizeof(clientData.len) + strlen(clientData.data) );
	//	printf("[TM::Send Data] [%d] [%d] [%s]\n", 
	//			clientData.type, clientData.len, clientData.data  );

	if( acha->m_bDebug )
	{
		printf("CM::local[%s] => %s[%s] 전송중..", lPath, remoteTarget, rPath);
	}

	// 데이터 전송
	int len;
	for( int amount = 0; amount < totalSize; )
	{
		memset( buffer, 0, sizeof(buffer) );

		len = read( fd, buffer, DATASIZ );
		if( len <= 0 )
		{
			break;
		}

		write( rtarget_sockfd, buffer, len );
		amount += len;

		sendAmount = amount;
	}

	close( fd );

	// 전송 결과 확인
	memset( &serverData, 0, sizeof(serverData) );
	read( rtarget_sockfd, &serverData, sizeof(serverData.type) + sizeof(serverData.len) );
	read( rtarget_sockfd, serverData.data, serverData.len );

	if( serverData.type == CM_ACK_OK )
	{
		if( acha->m_bDebug )
		{
			setcolor( COLOR_PINK );
			cout << " 완료!!" << endl;
			setcolor( COLOR_NONE );
		}
		return 0;
	}
	else
	{
		if( acha->m_bDebug ) cout << " 실패" << endl;
		return serverData.type;
	}
	if( acha->m_bDebug )
	{
		printf( "전송 결과 : [%d] [%d] [%s]\n", serverData.type, serverData.len, serverData.data );
	}
}

/*
 * 함수 : 파일을 전송 받는다
 * 반환 : 성공 : CM_ACK_OK
 *        실패 : CM_ACK_NO
 */
int CommunicationManager::receiveFile( void )
{
	char filePath[DATASIZ] = "";
	char hashCode[DATASIZ] = "";
	mode_t filePermission;

	// 전송 받을 파일에 관한 정보를 먼저 수신한다.
	memset( &serverData, 0, sizeof(serverData) );
	read( client_sockfd, &serverData,
			sizeof(serverData.type) + sizeof(serverData.len) );
	if( serverData.len > 0 )
	{
		read( client_sockfd, &serverData.data, serverData.len );
	}

//	sscanf( tm->serverData.data, "%s|%d|%s|%d", 
//			tm->filePath, &tm->totalSize, tm->hashCode, &filePermission );

	// 수신한 데이터를 파싱한다.
	char dataBuf[DATASIZ] = "";
	strncpy( dataBuf, serverData.data, DATASIZ );
	strcpy( filePath, strtok( dataBuf, "|" ) );
	totalSize = atoi( strtok( NULL, "|") );
	strcpy( hashCode, strtok( NULL, "|" ) );
	filePermission = (mode_t)atoi( strtok( NULL, "|\0") );

	// Temp 폴더가 없으면 만든다.
	if( access( TEMPFOLDER, F_OK | W_OK ) )
	{
		mkdir( TEMPFOLDER, (mode_t)0777 );
	}

	fpTemp = fopen( TMTEMP, "w" );
	if( fpTemp == NULL )
	{
		if( acha->m_bDebug )
		{
			cout << "TM::clientConnect() - tmp 파일 생성 실패" << endl;
		}
		return CM_ACK_NO;
	}

	if( acha->m_bDebug )
	{
		printf("[%s] 수신중..", filePath);
	}

	// 파일을 수신한다
	char buffer[DATASIZ];
	int len;
	for( int amount = 0; amount < totalSize; )
	{
		memset( buffer, 0, sizeof(buffer) );
		len = read( client_sockfd, buffer, DATASIZ );
		if( len <= 0 )
		{
			break;
		}

		fwrite( buffer, len, 1, fpTemp );
		amount += len;

		receiveAmount = amount;
	}

	fclose( fpTemp );

	if( strcmp( hashCode, md5file( TMTEMP ) ) == 0 )
	{
		if( acha->m_bDebug )
		{
			setcolor( COLOR_SKY );
			cout << " MD5 check.";
			setcolor( COLOR_NONE );
		}
	}
	else
	{
		if( acha->m_bDebug )
		{
			cout << "MD5 체크/파일 수신 실패" << endl;
		}
		return CM_ACK_NO;
	}

	// 백업 루틴 수행
	BackupFile( filePath );

	// temp 파일을 기본 파일로 변경
	if( rename( TMTEMP, filePath ) )
	{
		if( acha->m_bDebug )
		{
			printf( "파일 경로 변경 : %s -> %s -- 실패\n", TMTEMP, filePath );
		}
		return CM_ACK_NO;
	}
	if( acha->m_bDebug )
	{
		setcolor( COLOR_GREEN );
		cout << " 파일생성";
		setcolor( COLOR_NONE );
	}

	// 파일 퍼미션 변경
	if( chmod( filePath, filePermission ) )
	{
		if( acha->m_bDebug )
		{
			cout << "파일 권한 변경 -- 실패" << endl;
		}
		return CM_ACK_NO;
	}
	if( acha->m_bDebug )
	{
		setcolor( COLOR_YELLOW );
		cout << " 권한변경";
		setcolor( COLOR_NONE );
	}

	if( acha->m_bDebug )
	{
		setcolor( COLOR_PINK );
		cout << " 완료!!" << endl;
		setcolor( COLOR_NONE );
	}

	if( acha->m_bDebug )	cout << endl;

	return CM_ACK_OK;
}

bool CommunicationManager::BackupFile( char *sFilePath )
{
	SyncManager	*sm	= (SyncManager *)acha->m_pSyncManager;
	int nIndex;

	nIndex = sm->syncExist( sm->GetCurSyncName() );
	if( nIndex >= 0 ) {
		char *pLocalPath = sm->syncList[nIndex].localPath;
		char backupPath[SDATA];
		char sFileName[SDATA];
		char buf[SDATA];
		char *pch = NULL;

		// 백업 경로 찾아오기
		memset( backupPath, 0, SDATA );
		memset( buf, 0, SDATA );
		pch = strstr( sFilePath, pLocalPath );
		pch += strlen( pLocalPath );
		if( pLocalPath[strlen(pLocalPath)] != '/' )
		{
			pch++;
		}
		snprintf( buf, SDATA, "%s/%s/%s", 
				BACKUPDIR, sm->GetCurSyncName(), pch );

		pch = strrchr( buf, '/' );
		strncpy( sFileName, pch+1, SDATA );
		*pch = '\0';
		snprintf( backupPath, SDATA, "%s/%s_%s",
				buf, timeFormat(time(NULL)), sFileName );

		// 백업 경로까지 디렉토리가 없으면 만들어 준다
		memset( buf, 0, SDATA );
		strncpy( buf, backupPath, SDATA );
		pch = strrchr( buf, '/' );
		*pch = '\0';
		// 이제 buf에 남은건 디렉토리 경로 뿐..
		int len = (int)strlen(buf);
		for( int i = 0; i < len + 1; i++ )
		{
			if( '/' == buf[i] || '\0' == buf[i] )
			{
				buf[i] = '\0';
				// 해당 디렉토리가 존재하지 않으면..
				debug( buf, 999999 );
				if( 0 != access( buf, F_OK ) )
				{
					mkdir( buf, (mode_t)0755 );
				}
				buf[i] = '/';
			}
		}

		// 파일을 백업한다.
		if( rename( sFilePath, backupPath ) )
		{
			return false;
		}
	}

	return true;
}
