/*===================================//
 *
 * cm.h
 *
 * cm.cpp를 위한 헤더
 *
 * Written by JH. Cho.
 *
 *===================================*/
#ifndef __COMMUNICATION_MANAGER_H__
#define __COMMUNICATION_MANAGER_H__

#include "acha.h"

// 전송단위 메시지 구조체
typedef struct _cmdData {
	unsigned int type;
	unsigned int len;
	char data[DATASIZ];
} cmdData;

// 접속중인 클라이언트 정보
typedef struct _clientInfo {
	pthread_t tid;
	int clientSockFd;
	string clientIP;
} clientInfo;

void *listenThread( void * );	// 접속
void *acceptThread( void * );	// 접속 처리
void *timer( void * );			// 타이머

// Communication 메인 클래스
class CommunicationManager : public ACHABase
{
	private :
	public :
		// 소켓 관련변수
		int listenPort;
		int sendPort;
		int connectPort;
		int controlPort;

		int control_sockfd;				// 컨트롤 서버 소켓
		int server_sockfd;				// 서버 소켓
		int rtarget_sockfd;				// 원격지 서버 접속용 소켓
		int client_sockfd;				// 접속된 클라이언트 소켓

		struct sockaddr_in control_addr;// 컨트롤 서버 주소
		struct sockaddr_in server_addr;	// 서버 주소
		struct sockaddr_in client_addr;	// 클라이언트 주소
		struct sockaddr_in rtarget_addr;// 원격지 주소

		// 접속중인 클라이언트 주소
		struct sockaddr_in m_ConnectedClientAddr;

		socklen_t control_len;			// 컨트롤 서버 소켓 크기
		socklen_t server_len;			// 서버 소켓 크기
		socklen_t client_len;			// 클라이언트 소켓 크기
		socklen_t rtarget_len;			// 원격지 소켓 크기

		cmdData serverData;				// 서버 데이터 구조체
		cmdData serverResult;			// 서버 데이터 구조체
		cmdData clientData;				// 클라이지언트 데이터 구조체
		cmdData clientResult;			// 클라이지언트 데이터 구조체

		int m_rcReceive;				// 데이터 수신 후 결과
		int m_rcSend;					// 데이터 전송 후 결과

		char remoteTarget[SDATA];		// 접속할 원격지
		char m_sCurSyncName[SDATA];		// 현재 동기화 중인 SyncName
		list<clientInfo> clientList;	// 접속중인 클라이언트 리스트

		char cmdBuffer[DATASIZ];		// 수신버퍼 

	public :
		pthread_t	m_tid;				// acceptProc의 threadID
		pthread_t	m_tidAcceptThread;
		ACHA 		*acha;

		CommunicationManager(ACHA *pAcha);
		~CommunicationManager();

		bool m_bClientConnected;		// 접속중인 클라이언트가 있다

		int init( void );				// 초기화
		int cleanup( void );			// 정리
		int run( void );				// CM 실행 ListenThread 실행

		int  sendCmd( cmdData );				// 명령을 전송한다
		int  sendCmd( int );					// 명령을 전송한다
		int  sendCmd( int, char* );				// 명령을 전송한다
		bool makeCmd( cmdData&, int );			// 전송단위 명령을 만든다
		bool makeCmd( cmdData&, int, char* );	// 전송단위 명령을 만든다
		void SetTarget( char *sTarget );		// 접속할 원격지를 세팅한다.

		// 받은 명령어를 분석한다.
		// 분석한 명령은 처리하고 응답으로 던져줄 명령을 만들어 준다.
		int	 recvDataParsing( void *pt, cmdData& recv, cmdData& resp );

		void init_server( void );			// 소켓 초기화
		void bind_server( void );			// 포트할당
		void close_server( void );			// 소켓 종료

		void init_control_server( void );	// 컨트롤 서버 실행
		void close_control_server( void );	// 컨트롤 서버 실행 중지


		int init_client( char *, int );		// client를 초기화 한다.
		int connect_client( char *, int );	// Target, Port로 접속한다.
		int init_client( void );			// client를 초기화 한다.
		int connect_client( void );			// 원격지에 접속한다.
		int disconnect_client( void );		// 원격지와의 접속을 종료한다.

		int  sendCmd( void );				// 명령을 전송한다
		char *makeCmd( int );				// 전송단위 명령을 만든다
		char *makeCmd( int, char* );		// 전송단위 명령을 만든다

		bool invalidTarget( char * );		// 유효하지 않은 대상인지 확인
		void dispSettings( void );			// 현재 설정을 모두 출력함

		void portRefresh( void );			// 상위포트로이동
		int  reply( void );					// 현재 접속중인 클라이언트에 대답한다.

		/////////////////////////////////
		// Transfer Manager Function
		int totalSize;
		int sendAmount;
		int receiveAmount;

		FILE *fpTemp;

		int sendFile( char *, char *, char * );
		int receiveFile( void );
		bool BackupFile( char * );
};
#endif
