/*==============================================//
 *
 * fm.cpp
 *
 * 해당 폴더가 변화되었는지를 감시한다.
 *
 * SM의 지시를 받아서 동작한다.
 *
 * ============================================*/
#include "acha.h"
#include "fm.h"

FileManager::FileManager(ACHA *pAcha)
{
	acha		= pAcha;
	m_bIsRun	= false;
	scanList.reserve(65536);
	lastList.reserve(65536);
	changeList.reserve(65536);
}

FileManager::~FileManager(void)
{
}


// 초기화
int FileManager::init(void)
{
	lm.SetOutputMode(OUTPUT_FILE);
	lm.SetLogLevel(LV_INFO);
	lm.SetOwner("FM");
#ifdef TEST
	lm.SetLogLevel(LV_TRACE);
#endif

	return RC_OK;
}

// 초기화
int FileManager::run(void)
{

	return RC_OK;
}

// 정리
int FileManager::cleanup(void)
{
	return RC_OK;
}

// STL 관련 함수
bool CompareObj(scanInfo first, scanInfo second)
{
	if(strcmp(first.name, second.name) < 0)
		return true;
	else
		return false;
}

/**********************************************
 * 정렬을 위한 비교함수(STL 관련)
 * type 순으로 정렬한다.
 */
bool ComparePriority(changeInfo x, changeInfo y)
{
	if(x.type == y.type)
	{
		if(strcmp(x.lPath, y.lPath) < 0)
		{
			if(x.type == CI_DIRDEL)
				return false;

			return true;
		}
		else
		{
			if(x.type == CI_DIRDEL)
				return true;

			return false;
		}
	}
	if((x.type - y.type) < 0)
	{
		return true;
	}
	else
	{
		return false;
	}
}

// 폴더의 내용이 변화 되었는지 확인하는 함수
int FileManager::isChanged(syncListNode *synclistnode)
{
	char fileName[MAX_FILENAME];		// ChangedFileName

	// lastList 초기화
	while(lastList.size() > 0)
	{
		lastList.pop_back();
	}

	// CONFIGDIR :: config/syncName 내용을 읽어온다.
	bzero(fileName, sizeof(fileName));

	strcpy(fileName, CONFIGDIR);
	strcat(fileName, "/");
	strcat(fileName, synclistnode->syncName);

	loadChangedFile( fileName );	// 기존의 폴더 정보를 불러온다.

	// scanList 초기화 :: 새로 스켄할 내용을 저장할 구조체
	while(scanList.size() > 0)
	{
		scanList.pop_back();
	}

	// changeList 초기화 :: 변화된 내용 추가
	while(changeList.size() > 0)
	{
		changeList.pop_back();
	}

	// 새로 폴더 스캔.(전에 있던 폴더의 내용과 새로 스캔한 내용의 비교를 위함)
	scanSubDirectory(synclistnode->localPath, &synclistnode->lastSyncTime);

	// scanList 오름차순 정렬
	sort(scanList.begin(), scanList.end(), CompareObj);

	unsigned int scancnt = 0;
	unsigned int lastcnt = 0;
	int comp_result = 0;

	// 실제 비교가 일어나는 부분 - 이 부분에서 Changed List를 생성한다.
	changeInfo ci_element;

	while(scancnt < scanList.size() && lastcnt < lastList.size())
	{
		// 기존의 폴더 내용과 새로 스캔한 내용을 비교한다.
		comp_result = strcmp(scanList[scancnt].name, lastList[lastcnt].name);

		// 같을 경우에 파일 크기나 수정 날짜를 비교한다.
		if(comp_result == 0)
		{
			if(scanList[scancnt].mod_time > synclistnode->lastSyncTime)
			{
				strncpy(ci_element.lPath, scanList[scancnt].name, MAX_FILENAME);
				ci_element.type = (scanList[scancnt].type == DIR_DIR) ? CI_DIRADD : CI_FILEADD;

				strcpy(ci_element.rPath, gen_rpath(synclistnode->localPath, synclistnode->remotePath, ci_element.lPath));

				changeList.push_back(ci_element);	// changeList에 추가한다.
			}

			scancnt++;
			lastcnt++;
		}
		// 삭제 된 경우
		else if(comp_result > 0)
		{
			strncpy(ci_element.lPath, lastList[lastcnt].name, MAX_FILENAME);
			ci_element.type = (lastList[lastcnt].type == DIR_DIR) ? CI_DIRDEL : CI_FILEDEL;
			
			strcpy(ci_element.rPath, gen_rpath(synclistnode->localPath, synclistnode->remotePath, ci_element.lPath));

			changeList.push_back(ci_element);
			lastcnt++;
		}
		// 추가된 경우
		else
		{
// 추가 되었을 경우 마지막 Synctime 을 계산하여 재전송을 막는다.
//			if(scanList[scancnt].mod_time > synclistnode->lastSyncTime)
			{
				strncpy(ci_element.lPath, scanList[scancnt].name, MAX_FILENAME);
				ci_element.type = (scanList[scancnt].type == DIR_DIR) ? CI_DIRADD : CI_FILEADD;

				strcpy(ci_element.rPath, gen_rpath(synclistnode->localPath, synclistnode->remotePath, ci_element.lPath));

				changeList.push_back(ci_element);
			}
			scancnt++;
		}	
	}

	while(scancnt < scanList.size())
	{
		strncpy(ci_element.lPath, scanList[scancnt].name, MAX_FILENAME);
		ci_element.type = (scanList[scancnt].type == DIR_DIR) ? CI_DIRADD : CI_FILEADD;

		strcpy(ci_element.rPath, gen_rpath(synclistnode->localPath, synclistnode->remotePath, ci_element.lPath));

		changeList.push_back(ci_element);
		scancnt++;
	}

	while(lastcnt < lastList.size())
	{
		strncpy(ci_element.lPath, lastList[lastcnt].name, MAX_FILENAME);

		ci_element.type = (lastList[lastcnt].type == DIR_DIR) ? CI_DIRDEL : CI_FILEDEL;

		strcpy(ci_element.rPath, gen_rpath(synclistnode->localPath, synclistnode->remotePath, ci_element.lPath));

		changeList.push_back(ci_element);
		lastcnt++;
	}

	sort( changeList.begin(), changeList.end(), ComparePriority );

	return 0;
}

// 디버그를 위한 함수. Change List의 내용을 출력한다.
void FileManager::showList(int type)
{
	setcolor(COLOR_PINK);
//	puts("[============================ Change List =============================]");
	setcolor(COLOR_NONE);

	if(type == SHOW_SCAN)
	{
		for(unsigned int i = 0; i < scanList.size(); i++)
		{
			//printf("[1;34m Type : [00m%s [1;34mName :[00m %-50s[1;34m Time :[00m %s\n", scanList[i].type == DIR_DIR ? " [1;31mDIR[00m" : "[1;32mFILE[00m", scanList[i].name, timeFormat(scanList[i].mod_time));
			printf("%s %-50s[1;34m%s[00m\n", scanList[i].type == DIR_DIR ? " [1;31mDIR" : "[1;32mFILE[00m", scanList[i].name, timeFormat(scanList[i].mod_time));
		}
	}

	else if(type == SHOW_LAST)
	{
		for(unsigned int i = 0; i < lastList.size(); i++)
		{
			//printf("[1;34m Type : [00m%s [1;34mName :[00m %-50s[1;34m Time :[00m %s\n", scanList[i].type == DIR_DIR ? " [1;31mDIR[00m" : "[1;32mFILE[00m", scanList[i].name, timeFormat(scanList[i].mod_time));
			printf("%s %-50s[1;34m%s[00m\n", lastList[i].type == DIR_DIR ? " [1;31mDIR" : "[1;32mFILE[00m", lastList[i].name, timeFormat(lastList[i].mod_time));
		}
	}

	else if(type == SHOW_CHANGE)
	{
		for(unsigned int i = 0; i < changeList.size(); i++)
		{
			printf("%s %-20s %-20s[%d]\n", (changeList[i].type == CI_DIRADD || changeList[i].type == CI_FILEADD) ? "[1;31mCOPY[00m" : " [1;32mDEL[00m", changeList[i].lPath, changeList[i].rPath, changeList[i].type );
		}
	}

	return;
}

/******************************************
 * ChangedFile을 로드하는 함수
 */
void FileManager::loadChangedFile(char *path)
{
	FILE *fp;
	char buf[DATASIZ];

	if(access( CONFIGDIR, F_OK | W_OK ))
	{
		mkdir( CONFIGDIR, (mode_t)0777);
	}

	fp = fopen(path, "r+");

	if(fp == NULL)				// 기존의 ChangedFile이 없으면
	{
		fp = fopen(path, "w");
		
		if(fp == NULL)			// 새로운 ChangedFile 생성
		{
			lm.log(LV_ERROR, "loadChangedFile : 기존 changedFile없음 -> 파일생성 실패");
			return;
		}
		
		fclose(fp);
		return;
	}

	while( 1 ) // parsing :: 구분자 '|'
	{
		char *tok;
		bzero( buf, sizeof(buf) );
		fgets( buf, DATASIZ, fp );

		if ( !strlen(buf) )
		{
			break;
		}

		tok = strtok( buf, "|" );
		strcpy( element.name, tok );
		tok = strtok( NULL, "|" );

		int ttmp;
		sscanf( tok, "%d", &ttmp );
		element.mod_time = ttmp;

		tok = strtok( NULL, "|" );
		sscanf( tok, "%d", &element.type );

		lastList.push_back( element );
	}

	fclose(fp);
}

/*********************************************
 * ChangedFileList를 파일로 저장하는 함수
 */
void FileManager::writeChangedFile( char *path )
{
	FILE *fp;

	if( !access( CONFIGDIR, F_OK | W_OK ) )
	{
		mkdir( CONFIGDIR, (mode_t)0777 );
	}

	fp = fopen( path, "w");

	if( fp == NULL )
	{
		puts( "FileManager::writeChangedFile() 파일을 쓸수 없습니다.\n" );
		return;
	}

	for (unsigned int i = 0; i < scanList.size(); i++)
	{
		fprintf(fp, "%s|%d|%d|\n", scanList[i].name, (int)scanList[i].mod_time, scanList[i].type);
	}

	fclose(fp);

	return;
}

// sm에서 사용하는 함수
// syncName으로 부터 path를 생성하고, 해당 path에 changed List의 내용을 기록.
void FileManager::writeCFL( char *syncName )
{
	char path[MAX_FILENAME];

	bzero(path, sizeof(path));

	strcpy(path, CONFIGDIR);
	strcat(path, "/");
	strcat(path, syncName);

	writeChangedFile(path); 

	return;
}

// 폴더의 내용을 스캔하는 함수.
int FileManager::scanSubDirectory(char *target, time_t *lastsync)
{
	DIR *dpScan;

	dpScan = opendir( target );

	lm.log(LV_TRACE, "%s : Open the path - %s", __FUNCTION__, target);

	if(dpScan == NULL)
	{
		lm.log(LV_ERROR, "Can't open the path - %s", target);
		return -1;
	}

	while( true )
	{
		dp = readdir( dpScan );
		if( dp == NULL )
		{
			break;
		}

		// 숨김, 현재 및 상위 디렉토리 무시.
		if( dp->d_name[0] == '.' )
		{
			continue;
		}

		// 실제 데이터 출력 => 차후 데이터 클래스에 삽입
		element.type = DIR_FILE;
		strncpy(element.name, target, MAX_FILENAME);
		strcat(element.name, "/");
		strcat(element.name, dp->d_name);

		stat( element.name, &fs);
		element.mod_time = fs.st_mtime;

		// printf("%s\n", timeFormat(fs.st_mtime));
		scanList.push_back(element);

		//printf("%s (%s)\n", scanList[scanList.size() - 1].name, timeFormat(fs.st_mtime));

		if( dp->d_type == DIR_DIR )
		{			
			scanList[scanList.size() - 1].type = DIR_DIR;

			char *tmp = strdup( scanList[scanList.size()-1].name );
			scanSubDirectory( tmp, lastsync);
			free( tmp );
		}
	}

	closedir(dpScan);

	return 0;
}

// 폴더의 내용을 스캔하는 함수.
void FileManager::dispSettings(void)
{
	// 각 리스트마다 따로 찍을 것.
	return;
	setcolor( COLOR_SKY );
	cout << "[====================  FM Information  ====================]\n";
	setcolor( COLOR_NONE );
	cout << "[1m";
	showList(SHOW_CHANGE);
	cout << "[00m";
}

