#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>

#include "acha.h"
#include "lm.h"

#ifdef LMTEST
// 사용법.
int main()
{
	ACHALog er;

	er.SetOutputMode(OUTPUT_FILE);
	er.SetOwner("FM");
	er.SetLogLevel(LV_TRACE);
	er.log("hello %d, %s", 12, "world");
	er.log(LV_DEBUG, "debug Level");
	er.log(LV_TRACE, "trace Level");
	er.log(LV_INFO, "info Level");

	setcolor(COLOR_RED);
	puts("hello world");
	setcolor(COLOR_NONE);

	return 0;
}
#endif

//----------------------------------------------------------------------------------
// 멤버 변수 초기화.
//----------------------------------------------------------------------------------
ACHALog::ACHALog()
{
	m_pErrorFile		= NULL;
	m_pModuleLogFile	= NULL;
	m_szFileName[0]		= 0;
	m_nNow				= 0;
	m_szErrorString[0]	= 0;
	m_szOwner[0]		= 0;

	SetOutputMode(OUTPUT_NOT_SET);
	SetLogLevel(LV_INFO);
}

//----------------------------------------------------------------------------------
// 멤버 변수 초기화와 리소스 해제.
//----------------------------------------------------------------------------------
ACHALog::~ACHALog()
{
	m_szFileName[0] 	= 0;
	m_nNow				= 0;

	if( m_pErrorFile )
	{
		fclose( m_pErrorFile );
		m_pErrorFile = NULL;
	}
	if( m_pModuleLogFile )
	{
		fclose( m_pModuleLogFile );
		m_pModuleLogFile = NULL;
	}

	m_szErrorString[0] 	= 0;
	m_nOutputMode	   	= OUTPUT_NOT_SET;
	m_szOwner[0] 		= 0;
}

//----------------------------------------------------------------------------------
// 새로운 파일을 열어야 하는지 검사고 열어야 하면 연다.
//----------------------------------------------------------------------------------
bool ACHALog::CheckNeedOpenNewFileAndFilePointer()
{
	char szPath[ERRORLOG_DIR_LEN];
	char szFilename[ERRORLOG_FILE_LEN];
	char szModulename[ERRORLOG_FILE_LEN];

	memset( szModulename, 0, ERRORLOG_FILE_LEN );

	strftime( szFilename, ERRORLOG_FILE_LEN, ERRORLOG_FILE_FORMAT, m_tmNow );
	strncat( szFilename, ".log", 4 );
	strncpy( szModulename, m_szOwner, strlen(m_szOwner) );
	strncat( szModulename, szFilename, strlen(szFilename) );

	// 만약 파일이 열려져있지 않고 파일의 이름이 변경되어 새로이 파일을 열어야하면?
	if( !strcmp( szFilename, m_szFileName ) || m_pErrorFile == NULL )
	{
		// 파일이 열려 있으면 닫는다.
		if( m_pErrorFile != NULL )
		{
			fclose( m_pErrorFile );
			m_pErrorFile = NULL;
		}

		strncpy( szPath, ".log", 5 );

		// Log 디렉토리를 생성한다.
		mkdir( szPath, S_IFDIR | S_IRWXU );

		strncat( szPath, "/", 1 );
		strncat( szPath, szFilename, strlen( szFilename ) );

//	debug
//		printf("path = %s\n", szPath);

		// 파일을 연다.
		if( ( m_pErrorFile = fopen( szPath, "a+" ) ) == NULL )
		{
			return false;
		}

		strncpy( szFilename, m_szFileName, strlen( szFilename )+1 );
	}

	// 만약 파일이 열려져있지 않고 파일의 이름이 변경되어 새로이 파일을 열어야하면?
	if( !strcmp( szModulename, m_szModuleName ) || m_pModuleLogFile == NULL )
	{
		// 파일이 열려 있으면 닫는다.
		if( m_pModuleLogFile != NULL )
		{
			fclose( m_pModuleLogFile );
			m_pModuleLogFile = NULL;
		}

		strncpy( szPath, ".log", 5 );
		strncat( szPath, "/", 1 );
		strncat( szPath, szModulename, strlen( szModulename ) );
	
//	debug
//		printf("szModulename = %s\n", szModulename);
//		printf("path = %s\n", szPath);

		// 파일을 연다.
		if( ( m_pModuleLogFile = fopen( szPath, "a+" ) ) == NULL )
		{
			return false;
		}

		strncpy( szModulename, m_szModuleName, strlen( szModulename )+1 );
	}
	return true;
}

//----------------------------------------------------------------------------------
// 현재 시간 멤버 변수를 설정한다.
//----------------------------------------------------------------------------------
void ACHALog::SetCurrentTime()
{
	time( &m_nNow );
	m_tmNow = localtime( &m_nNow );
}

//----------------------------------------------------------------------------------
// 에러 출력하기.
//----------------------------------------------------------------------------------
void ACHALog::log( LOG_LEVEL enumLevel, const char * szErrorMsg, ... )
{
	char *arLogLevel[] = {
		"TRACE",
		"DEBUG",
		"INFO",
		"ERROR",
		"BUG",
		"CRITICAL"
	};

	if( enumLevel < m_enumLogLevel )
		return;

	if( szErrorMsg == NULL || m_nOutputMode == OUTPUT_NOT_SET )
		return;

	va_list Args;
	char szTimeString[TIME_STRING_LEN];
	int nLen = 0;

	SetCurrentTime();

	// 시간을 설정된 포멧에 마추어 문자열로 만든다.
    if( !( m_tmNow ) )
		strncpy( szTimeString, "?", 2 );
    else
		strftime( szTimeString, TIME_STRING_LEN, TIME_STRING_FORMAT, m_tmNow );

	nLen = sprintf( m_szErrorString,"[%-4s][%s][%s] : ",
					m_szOwner, szTimeString, arLogLevel[enumLevel] );

	// 가변인자 받기.
	va_start( Args, szErrorMsg );
	vsprintf( m_szErrorString+nLen, szErrorMsg, Args );
	va_end( Args );

	// 모드에 마추어 출력한다.
#ifdef WIN32
	if( m_nOutputMode & OUTPUT_MESSAGEBOX )
	{
		OutputMessageBox();
	}
#endif
	if( m_nOutputMode & OUTPUT_FILE )
	{
		OutputFile();
	}
	if( m_nOutputMode & OUTPUT_CONSOLE )
	{
		OutputConsole();
	}
}

void ACHALog::log( const char * szErrorMsg, ... )
{
	if( LV_INFO < m_enumLogLevel )
		return;
	if( szErrorMsg == NULL || m_nOutputMode == OUTPUT_NOT_SET )
		return;

	va_list Args;
	char szTimeString[TIME_STRING_LEN];
	int nLen = 0;

	SetCurrentTime();

	// 시간을 설정된 포멧에 마추어 문자열로 만든다.
    if( !( m_tmNow ) )
		strncpy( szTimeString, "?", 2 );
    else
		strftime( szTimeString, TIME_STRING_LEN, TIME_STRING_FORMAT, m_tmNow );

	nLen = sprintf( m_szErrorString, "[%s][%s] : ", m_szOwner, szTimeString );

	// 가변인자 받기.
	va_start( Args, szErrorMsg );
	vsprintf( m_szErrorString+nLen, szErrorMsg, Args );
	va_end( Args );

	// 모드에 마추어 출력한다.
	if( m_nOutputMode & OUTPUT_FILE )
	{
		OutputFile();
	}
	if( m_nOutputMode & OUTPUT_CONSOLE )
	{
		OutputConsole();
	}
}

//----------------------------------------------------------------------------------
// 파일에 출력.
//----------------------------------------------------------------------------------
void ACHALog::OutputFile()
{
	if( !CheckNeedOpenNewFileAndFilePointer() )
		return;

	if( m_pErrorFile != NULL )
	{
		fprintf( m_pErrorFile, "%s\n", m_szErrorString );
		fflush( m_pErrorFile );
	}

	if( m_pModuleLogFile != NULL )
	{
		fprintf( m_pModuleLogFile, "%s\n", m_szErrorString );
		fflush( m_pModuleLogFile );
	}
}

//----------------------------------------------------------------------------------
// Console 모드 프로젝트일때 화면에 출력한다.
//----------------------------------------------------------------------------------
void ACHALog::OutputConsole()
{
	printf( "%s\n", m_szErrorString );
}

void ACHALog::SetOwner( char *szOwner )
{
	int length = 0;

	length = (strlen(szOwner) < ERRORLOG_FILE_LEN) ? strlen(szOwner) : ERRORLOG_FILE_LEN ;

// 	debug
//	printf("%s, %d, %d\n", szOwner, strlen(szOwner), length);

	memset(m_szOwner, 0, ERRORLOG_FILE_LEN);
	strncpy(m_szOwner, szOwner, length );

	return;
}
