#ifndef __ACHALog_H__
#define __ACHALog_H__

#define ERRORLOG_FILE_FORMAT	"%Y%m%d"
#define TIME_STRING_FORMAT		"%H:%M:%S"

//! Output mode
enum OUTPUT_MODE
{
	OUTPUT_NOT_SET				= 0,
	OUTPUT_FILE					= 1,
	OUTPUT_CONSOLE				= 2,
	OUTPUT_FILE_AND_CONSOLE		= OUTPUT_FILE | OUTPUT_CONSOLE,
	OUTPUT_ALL					= OUTPUT_FILE | OUTPUT_CONSOLE
};

// 해당 레벨 이상의 로그만 출력
enum LOG_LEVEL
{
	LV_TRACE,
	LV_DEBUG,
	LV_INFO,
	LV_ERROR,
	LV_BUG,
	LV_CRITICAL
};

class ACHALog
{
	public:
		//! 클래스에서 사용되는 변수들의 길이.
		enum
		{
			/*! 로그가 출력될 디렉토리의 최대 길이. */
			ERRORLOG_DIR_LEN	= 30,
			/*! 로그 파일의 최대 길이. */
			ERRORLOG_FILE_LEN	= 20,
			/*! 시간 문자열의 최대 길이. */
			TIME_STRING_LEN		= 30,
			/*! 기록할 로그 문자열의 최대 길이. */
			ERRORLOG_STRING_LEN = 512
		};


	private:
		FILE *		m_pErrorFile;				// 통합 로그파일 포인터
		FILE *		m_pModuleLogFile;			// 모듈별 로그파일 포인터
		char		m_szFileName[ERRORLOG_FILE_LEN];
		char		m_szModuleName[ERRORLOG_FILE_LEN];
		//! variable for get current time
		struct tm * m_tmNow;
		//! variable for get current time
		time_t		m_nNow;
		//! error string.
		char		m_szErrorString[ERRORLOG_STRING_LEN];
		//! Output mode.
		int			m_nOutputMode;
		//! 로그를 찍는 주체
		char		m_szOwner[ERRORLOG_FILE_LEN];	
		//! 로그 레벨
		LOG_LEVEL	m_enumLogLevel;	

	public:
		//! 생성자.
		/*! 멤버 변수를 초기화 한다. */
		ACHALog();
		//! 소멸자.
		/*! 멤버 변수를 초기화와 리소스 해제를 한다. */
		~ACHALog();

		//! Output Mode를 설정한다.
		void SetOutputMode( int nOutputMode = OUTPUT_FILE ) { m_nOutputMode = nOutputMode; }

		//! ErrorLog 문자열 구하기.
		char * GetErrorString() { return m_szErrorString; }

		//! 에러 출력하기.
		void log( const char * szErrorMsg, ... );
		void log( LOG_LEVEL enumLevel, const char * szErrorMsg, ... );

		//! 로그를 출력하는 주체를 설정한다.
		void SetOwner( char * );
		//! 로그 레벨을 설정한다.
		void SetLogLevel( LOG_LEVEL enumLevel ) { m_enumLogLevel = enumLevel; };

	private:
		//! 새로운 파일을 열어야 하는지 검사고 열어야 하면 연다.
		bool CheckNeedOpenNewFileAndFilePointer();
		//! 현재 시간 멤버 변수를 설정한다.
		void SetCurrentTime();

#ifdef WIN32 // WINDOWS
		//! MessageBox 출력.
		void OutputMessageBox();
#endif
		//! 파일에 출력.
		void OutputFile();
		//! Console 모드 프로젝트 일때 화면에 출력.
		void OutputConsole();
};
#endif
