/*===================================//
 * sm.cpp
 * SyncManger를 위한 프로그램
 * 동기화 관련 루틴을 처리한다.
 * Written by JH. Cho.
 *===================================*/
#include "acha.h"
#include "sm.h"
#include "cm.h"
#include "fm.h"
#include "uim.h"

// 생성자
SyncManager::SyncManager(ACHA *pAcha)
{
	acha = pAcha;
}

// 소멸자
SyncManager::~SyncManager()
{
}

void SyncManager::test( void )
{
	cout << "compile ok!!" << endl;
}

/*
 * 함수 : SM을 초기화 한다.
 * 기능 : FM, CM, SM, UI thread 를 초기화, 실행 시킨다.
 * 리턴 : 성공할 경우 RC_OK를 반환한다
 */
int SyncManager::init(void)
{
	lm.SetOutputMode(OUTPUT_FILE);
	lm.SetLogLevel(LV_TRACE);
	lm.SetOwner("SM");

#ifdef TEST
	lm.SetLogLevel(LV_TRACE);
//	lm.SetOutputMode(OUTPUT_FILE|OUTPUT_CONSOLE);
#endif

	return RC_OK;
}

/*
 * 함수 : SM을 초기화 한다.
 * 기능 : FM, CM, SM, UI thread 를 초기화, 실행 시킨다.
 * 리턴 : 성공할 경우 RC_OK를 반환한다
 */
int SyncManager::run(void)
{
	int status;
    status = pthread_create( &m_tid, NULL, SyncManager::syncProc, acha);
    if( status < 0 ) 
    {   
        lm.log(LV_TRACE, "SM Thread 생성 실패");
    }   
    else
    {   
        lm.log(LV_TRACE, "SM Thread 생성 성공 : tid = %d", m_tid);
    }

	return RC_OK;
}

/*
 * 함수 : SM을 초기화 한다.
 * 기능 : FM, CM, SM, UI thread 를 초기화, 실행 시킨다.
 * 리턴 : 성공할 경우 RC_OK를 반환한다
 */
int SyncManager::cleanup(void)
{
	return RC_OK;
}

/************************************
 * 함수 : SyncList(File)를 로드하는 함수
 * 설명 : synclist 파일의 형식 
 *        SuncName | IP or Domain | localPath | remotePath |
 *        userId | password | flag | lastSyncTime
 *        위와 같은 형식의 파일을 읽어 syncList(Memory) 에 저장한다.
 * 참고 : syncList( STL의 dequeue형 )
 * ************************************/
void SyncManager::loadSyncList(ACHA *pAcha)
{
	SyncManager *sm = (SyncManager *)pAcha->m_pSyncManager;

    if( access( CONFIGDIR, F_OK | W_OK ) ) 			// 폴더존재여부 확인
    {   
        mkdir( CONFIGDIR, (mode_t)0777 );			// 없으면 폴더생성
    }   

    char buf[DATASIZ];
	
    sm->fpSyncList = fopen( SYNCLISTPATH, "r+" );		// 파일 open

	if( fpSyncList == NULL )						// 기존Sync규칙이 없으면
    {   
        puts("Sync 규칙이 만들어지지 않았습니다.");
        puts("새로운 SyncList 파일을 생성합니다.");
        sm->fpSyncList = fopen( SYNCLISTPATH, "w" );	// 파일 생성후 종료
        fclose( sm->fpSyncList );
        return;
    }   

    cleanSyncList(pAcha);							// SyncList 메모리 초기화

    while( 1 ) 
    {   
        char *tok;
        bzero( buf, sizeof(buf) );					// 버퍼를 '\0'으로 초기화
        fgets( buf, DATASIZ, sm->fpSyncList );			// 파일로부터 버퍼만큼 읽어

        if( !strlen(buf) )							// 읽은 문자가 없으면
        {
            break;									// 종료
        }

        tok = strtok( buf, "|" );					// SyncList File parsing
        slnTmp.syncName = strdup( tok );
        tok = strtok( NULL, "|" );
        slnTmp.target = strdup( tok );
        tok = strtok( NULL, "|" );
        slnTmp.localPath = strdup( tok );
        tok = strtok( NULL, "|" );
        slnTmp.remotePath = strdup( tok );
        tok = strtok( NULL, "|" );
        slnTmp.userId = strdup( tok );
		tok = strtok( NULL, "|" );
		slnTmp.password = strdup( tok );
		tok = strtok( NULL, "|" );
		sscanf( tok, "%d", &slnTmp.fileSize );
		tok = strtok( NULL, "|" );
		sscanf( tok, "%d", &slnTmp.option );
		tok = strtok( NULL, "|" );
        sscanf( tok, "%d", &slnTmp.Flag );

        int ttmp;
        tok = strtok( NULL, "|\n" );
        sscanf( tok, "%d", &ttmp );					// 문자 -> int형 변수
        
		slnTmp.lastSyncTime = (time_t)ttmp;

        sm->syncList.push_back( slnTmp );				// SyncList에 추가
    }

    fclose( sm->fpSyncList );

    return;
}

/* 함수 : syncList를 초기화 하는 함수
 * 설명 : dequeue에 있는 syncListNode를 가져와서 메모리를 해제한 후
 * 	      dequeue에 있는 내용을 pop 한다.
 */
void SyncManager::cleanSyncList(ACHA *pAcha)
{
	SyncManager *sm = (SyncManager *)pAcha->m_pSyncManager;

    syncListNode tmp;
    while( !sm->syncList.empty() )	// syncList에 있는 모든 내용 삭제
    {
        tmp = sm->syncList.front();
		cleanSyncListNode(tmp);
        sm->syncList.pop_front();
    }

    return;
}

/* 함수 : syncListNode한개의 메모리를 해제하는 함수
 * 설명 : 동적으로 할당된 메모리를 반환한다.
 */
void SyncManager::cleanSyncListNode( syncListNode tmp )
{
	delete tmp.syncName;
    delete tmp.target;
    delete tmp.localPath;
    delete tmp.remotePath;
	delete tmp.userId;
	delete tmp.password;
}

/* 함수 : syncList(Memory)에 있는 내용을 파일로 쓰는 함수
 * 주의 : 기존에 파일이 존재한다면 기존의 파일은 삭제되고 새롭게 생성된다.
 */
void SyncManager::writeSyncList( void )
{
    if( access( CONFIGDIR, F_OK | W_OK ) != 0 )	// config dir check 및 생성
    {
        mkdir( CONFIGDIR, (mode_t)0777 );
    }

    fpSyncList = fopen( SYNCLISTPATH, "w" );// 파일이 존재하면 기존의 파일은 삭제
    if( fpSyncList == NULL )
    {
        puts("writeSyncList() SyncList 파일을 쓸 수 없습니다.");
        return;
    }

    for( int i = 0; i < (int)syncList.size(); i++ )
    {
        fprintf( fpSyncList, "%s|%s|%s|%s|%s|%s|%d|%d|%d|%d|\n",
                syncList[i].syncName,
                syncList[i].target,
                syncList[i].localPath,
                syncList[i].remotePath,
				syncList[i].userId,
				syncList[i].password,
				syncList[i].fileSize,
				syncList[i].option,
                syncList[i].Flag,
                (int)syncList[i].lastSyncTime);

    }

    fclose( fpSyncList );
    return;
}

/* 함수 : 해당 이름의 Sync가 존재하는 지 확인
 * 반환 : 존재 index, 없음 -1
 */
int SyncManager::syncExist( char *name )
{
    for( unsigned int i = 0; i < syncList.size(); i++ )
    {
        if( strcmp( syncList[i].syncName, name ) == 0 )
        {
            return i;
        }
    }
    return -1;
}

/*
 * 함수 : 새로운 sync를  생성하는 함수
 */
int SyncManager::addSync( char *syncname, char *target, char *localpath, char *remotepath, int flag,
						  char *userId="id", char *password="pass", int filesize=1000000 )
{
    if( syncExist( syncname ) >= 0 )
    {
		lm.log(LV_INFO, "addSync() - Sync생성 실패 : %s 이름의 싱크가 존재합니다.", syncname);
        return RC_NO;
    }
	
	if( access( localpath, F_OK | R_OK ) )
	{
		lm.log(LV_INFO, "addSync() - Sync생성 실패 : 존재하지 않는 디렉토리");
		return RC_NO;
	}

	slnTmp.syncName     = strdup( syncname );
	slnTmp.target       = strdup( target );
	slnTmp.localPath    = strdup( localpath );
	slnTmp.remotePath   = strdup( remotepath );
	slnTmp.userId		= strdup( userId );
	slnTmp.password		= strdup( password );
	slnTmp.fileSize		= filesize;
	slnTmp.option		= 0;

	slnTmp.Flag 		= flag;
	slnTmp.lastSyncTime = 0;				

	syncList.push_back( slnTmp );

	if( !access( CONFIGDIR, F_OK | W_OK ) ) 			// 폴더존재여부 확인
	{   
		mkdir( CONFIGDIR, (mode_t)0777 );			// 없으면 폴더생성
	}   

	// Sync name 의 lastList 가 들어갈 파일을 생성(빈파일)
	char buf[DATASIZ] = "";
	sprintf( buf, "%s/%s", CONFIGDIR, syncname );
	fclose( fopen( buf, "w" ) );	
	
     writeSyncList();
    return RC_OK;
}


/* 함수 : 디버그를 위해서 syncList의 내용을 모두 출력하는 함수 
 */
void SyncManager::showList( void )
{
	setcolor( COLOR_SKY );
	printf("[=============================== SyncList ================================]\n");
	setcolor( COLOR_DARKRED );
	cout << "[SyncName]";
	setcolor( COLOR_DARKGREEN );
	cout << "[  Target  ]";
	setcolor( COLOR_BROWN );
	cout << "[  LocalPath  ]";
	setcolor( COLOR_PUPLE );
	cout << "[  RemotePath  ]"; 
	setcolor( COLOR_NONE );
	cout << "[Flag]";
	setcolor( COLOR_DARKSKY );
	cout << "[ LastSyncTime ]" << endl;
	for(int i = 0; i < (int)syncList.size(); i++)
	{
		setcolor( COLOR_DARKRED );
		printf("[%s]", syncList[i].syncName);
		setcolor( COLOR_DARKGREEN );
		printf("[%s]", syncList[i].target);
		setcolor( COLOR_BROWN );
		printf("[%s]", syncList[i].localPath);
		setcolor( COLOR_PUPLE );
		printf("[%s]", syncList[i].remotePath);
		//printf("[%s]", syncList[i].userId);
		//printf("[%s]", syncList[i].password);
		//printf("[%d]", syncList[i].fileSize);
		//printf("[%d]", syncList[i].option);
		setcolor( COLOR_NONE );
		printf("[%d]", syncList[i].Flag);
		setcolor( COLOR_DARKSKY );
		//printf("[%d]\n", (int)syncList[i].lastSyncTime);
		printf("[%s]\n", timeFormat(syncList[i].lastSyncTime) );
		setcolor( COLOR_NONE );
	}
}

/* 함수 : 디버그를 위해서 syncList의 내용을 모두 출력하는 함수 
 */
void SyncManager::dispSettings( void )
{   
	setcolor( COLOR_SKY );
	cout << "[====================  SM Information  ====================]\n";
	setcolor( COLOR_DARKRED );
	cout << "[SyncName]";
	setcolor( COLOR_DARKGREEN );
	cout << "[Target]";
	setcolor( COLOR_BROWN );
	cout << "[LocalPath]";
	setcolor( COLOR_PUPLE );
	cout << "[RemotePath]"; 
	setcolor( COLOR_NONE );
	cout << "[Flag]";
	setcolor( COLOR_DARKSKY );
	cout << "[LastSyncTime]" << endl;
	for(int i = 0; i < (int)syncList.size(); i++)
	{
		setcolor( COLOR_DARKRED );
		printf("[%s]", syncList[i].syncName);
		setcolor( COLOR_DARKGREEN );
		printf("[%s]", syncList[i].target);
		setcolor( COLOR_BROWN );
		printf("[%s]", syncList[i].localPath);
		setcolor( COLOR_PUPLE );
		printf("[%s]", syncList[i].remotePath);
		//printf("[%s]", syncList[i].userId);
		//printf("[%s]", syncList[i].password);
		//printf("[%d]", syncList[i].fileSize);
		//printf("[%d]", syncList[i].option);
		setcolor( COLOR_NONE );
		printf("[%d]", syncList[i].Flag);
		setcolor( COLOR_DARKSKY );
		//printf("[%d]\n", (int)syncList[i].lastSyncTime);
		printf("[%s]\n", timeFormat(syncList[i].lastSyncTime) );
		setcolor( COLOR_NONE );
	}
	setcolor( COLOR_NONE );
}

// 함수 : 해당하는 이름의 sync를 삭제한다.
// 설명 : 플래그만 세팅해 둔다.
//        스레드가 돌면서 원격지에도 삭제를 하면 삭제가 된다.
int SyncManager::delSync( char *syncName )
{
	setFlag( syncName, FLAG_DELSYNC );
	return 1;
}
// 함수 : 해당하는 이름의 sync를 삭제한다.
// 설명 : 리스트에서 바로 삭제한다. 
int SyncManager::delSyncNode( char *syncName )
{
	int  indexOfSync;
	int  sizeOfSyncList;
	int  nFlag;
	char buf[DATASIZ] = "";

	indexOfSync = syncExist( syncName );		// syncName 을 갖는 레코드 검색
	if ( indexOfSync < 0 )						// syncName이 존재하지 않으면
	{
		lm.log(LV_INFO, "존재하지 않는 syncName 입니다.");
		return RC_NO;
	}

	sizeOfSyncList 	= syncList.size();			// syncList Size 만큼 돈다
	nFlag 			= syncList[indexOfSync].Flag;	// 플래그는 일단 저장

	for( int i = 0; i < sizeOfSyncList; i++ )
	{
		slnTmp = syncList.front();				// syncList 처음부터 가져와서
		syncList.pop_front();

		if(indexOfSync != i )					// syncName을 갖는 레코드를 제외하곤
			syncList.push_back(slnTmp);			// 다시 syncList deque에 넣는다.
		else
			cleanSyncListNode(slnTmp);			// 같은 syncName을 갖는 레코드의 메모리 해제
	}
	writeSyncList();

	// LastList를 보관하고 있던 파일을 삭제한다.
	if( FLAG_SYNC == nFlag )
	{
		sprintf( buf, "%s/%s", CONFIGDIR, syncName );
		unlink( buf );
	}

	return RC_OK;
}

/* 함수 : syncList의 syncName에 해당하는 레코드의 syncTime을 update 한다.
 * 인자 : update 시킬 레코드의 syncName, 바꿀 syncTime(동기화 시킨 Time)
 * 주의 : syncName이 없을 경우를 고려하지 않기 때문에 반드시 함수 호출전에
 * 		  syncName이 존재하는지 확인하자
 */
int SyncManager::updateSyncTime(char *syncName, time_t lastSyncTime)
{
	int indexOfSync;
	indexOfSync = syncExist(syncName);

	syncList[indexOfSync].lastSyncTime = lastSyncTime;
	writeSyncList();

	return RC_OK;
}

/* 함수 : syncList의 syncName에 해당하는 레코드의 FileSize를 update 하는 함수
 * 인자 : update 시킬 레코드의 syncName, 바꿀 fileSize(제한시킬 크기)
 */
void SyncManager::updateFileSize(char *syncName, int fileSize)
{
	int indexOfSync;
	indexOfSync = syncExist(syncName);

	syncList[indexOfSync].fileSize = fileSize;
	writeSyncList();
}

/* 함수 : syncList의 syncName에 해당하는 레코드의 option을 update 하는 함수
 * 인자 : update 시킬 레코드의 syncName, 바꿀 option
 */
void SyncManager::updateOption(char *syncName, int option)
{
	int indexOfSync;
	indexOfSync = syncExist(syncName);

	syncList[indexOfSync].option = option;
	writeSyncList();
}

/* 함수 : SyncList의 Flag 필드를 다루는 함수
 * 인자 : update 시킬 레코드의 syncName 바꿀 플래그 옵션
 */
void SyncManager::setFlag(char *syncName, int flag)
{
	int indexOfSync;
	indexOfSync = syncExist(syncName);

	if( FLAG_DELSYNC == flag ) 
	{
		// 아직 원격지에 리스트가 추가 되기 전이면 바로 삭제
		if( syncList[indexOfSync].Flag == FLAG_NEWSYNC ) 
		{
			delSyncNode( syncList[indexOfSync].syncName );
		}
		else
		{
			syncList[indexOfSync].Flag = flag;
		}
	}

/*
	switch(flag)
	{
	case FLAG_ON_ADD:
		syncList[indexOfSync].Flag |= FLAG_ADD;
		break;
	case FLAG_OFF_ADD:
		syncList[indexOfSync].Flag ^= FLAG_ADD;
		break;
	case FLAG_ON_DELETE:
		syncList[indexOfSync].Flag |= FLAG_DELETE;
		break;
	case FLAG_OFF_DELETE:
		syncList[indexOfSync].Flag ^= FLAG_DELETE;
		break;
	case FLAG_ON_CHANGE_OPTION:
		syncList[indexOfSync].Flag |= FLAG_CHANGE_OPTION;
		break;
	case FLAG_OFF_CHANGE_OPTION:
		syncList[indexOfSync].Flag ^= FLAG_CHANGE_OPTION;
		break;
	case FLAG_ON_CHANGE_FILESIZE:
		syncList[indexOfSync].Flag |= FLAG_CHANGE_FILESIZE;
		break;
	case FLAG_OFF_CHANGE_FILESIZE:
		syncList[indexOfSync].Flag ^= FLAG_CHANGE_FILESIZE;
	}
*/
	writeSyncList();
}

/* 함수 : syncName을 갖는 해당 syncList의 Flag가 set되어 있는지 확인하는 함수
 * 인자 : syncName, flag: 확인할 flag
 */
bool SyncManager::isFlag(char *syncName, int flag)
{
	int indexOfSync;
	indexOfSync = syncExist( syncName );
	if( (syncList[indexOfSync].Flag & flag) != 0 )
		return true;

	return false;
}

/*
 * syncProc
 * 동기화와 관련된 로직을 가진다
 */
void *SyncManager::syncProc( void *pthis )
{
	ACHA					*acha 	= (ACHA *)pthis;
	SyncManager				*sm 	= (SyncManager *)acha->m_pSyncManager;
	CommunicationManager 	*cm 	= (CommunicationManager *)acha->m_pCommunicationManager;

	int  nRet;
	char szArgBuf[SDATA];

	sm->loadSyncList(acha);

	while( true )
	{
		for( int i = 0; i < (int)sm->syncList.size(); i++ )
		{
			if( true != acha->m_bLock )
			{
				sm->SetCurSyncName(sm->syncList[i].syncName);
				// SyncList 의 플래그가 Del 혹은 New 일 경우
				if ( sm->syncList[i].Flag == FLAG_DELSYNC || sm->syncList[i].Flag == FLAG_NEWSYNC )
				{
					// 타겟에 접속한다.
					nRet = cm->connect_client( sm->syncList[i].target, cm->sendPort );
					if( nRet != RC_OK )
					{
						sm->lm.log(LV_INFO, "Remote Connect Fail : %s", sm->syncList[i].target );
						cm->disconnect_client();
						continue;
					}


					// lock 실행 요청
					acha->lock();
					nRet = cm->sendCmd( CM_LOCK );
					if( CM_ACK_OK != nRet )
					{
						sm->lm.log(LV_INFO, "Remote Lock Fail : %s", sm->syncList[i].target );
						acha->unlock();
						continue;
					}

					if( sm->syncList[i].Flag == FLAG_NEWSYNC )
					{
						// AddSync 명령을 전송
						bzero( szArgBuf, sizeof(szArgBuf) );
						sprintf( szArgBuf, "%s %s %s", 
								sm->syncList[i].syncName, 
								sm->syncList[i].localPath, 
								sm->syncList[i].remotePath );
						nRet = cm->sendCmd( CM_ADD_SYNC, szArgBuf );
						if( CM_ACK_OK == nRet )
						{
							sm->syncList[i].Flag = FLAG_SYNC;
							sm->writeSyncList();
						}
						else
						{
							// 리턴값 처리할 것
							if( 1 ) // SyncName이 이미 있는 것일 경우
							{
								sm->lm.log(LV_INFO, "Sync Name is Exist");
								sm->delSync( sm->syncList[i].syncName );
							}

							sm->lm.log(LV_INFO, "Add Sync Fail : %s", sm->syncList[i].syncName);
						}
					}
					else if( sm->syncList[i].Flag == FLAG_DELSYNC )
					{
						// Sync 삭제 요청 
						nRet = cm->sendCmd( CM_DEL_SYNC, sm->syncList[i].syncName );
						if( CM_ACK_OK == nRet )
						{
							// 로컬 싱크 삭제
							sm->delSyncNode( sm->syncList[i].syncName );
							sm->writeSyncList();
						}
						else
						{
							sm->lm.log(LV_INFO, "Del Sync Fail : %s", sm->syncList[i].syncName);
						}
					}

					// Lock을 해제한다.
					cm->sendCmd( CM_UNLOCK );
					acha->unlock();

					// 접속을 끊는다.
					cm->disconnect_client();
					continue;
				}
				// SyncList 의 플래그가 Sync 일 경우
				else
				{
					sm->syncProcess( acha, i );
				}
			}
			else
			{
				while( acha->m_bLock ) {
					sleep(1);
				}
				i = 0;
				break;
			} // if( true != acha->m_bLock )
		} // for( int i = 0; i < (int)sm->syncList.size(); i++ )

		// Sync List를 모두 돌고 난 뒤 1초 쉰다
		sleep(1);

	} // while( true )

	return 0;
}

/*
 * 함수 : 동기화 프로세스
 * 설명 : 동기화 작업 중 파일 또는 폴더의 추가 삭제부분의 모듈이다.
 */
int SyncManager::syncProcess( void *pthis, int index )
{
	ACHA					*acha 	= (ACHA *)pthis;
	SyncManager				*sm 	= (SyncManager *)acha->m_pSyncManager;
	CommunicationManager 	*cm 	= (CommunicationManager *)acha->m_pCommunicationManager;
 	FileManager				*fm 	= (FileManager *)acha->m_pFileManager;

	int  nRet;
	char *lPath, *rPath;
	bool bUpdateTime 	 = true;
	char szBuf[DATASIZ]  = "";
	char md5tmp[DATASIZ] = "";

	int  i = index;
	// 변경된 내역이 있는지 확인
//	cout << "fm::isChanged " << sm->syncList[i].localPath << endl;
//	fm->isChanged( sm->syncList[i].localPath, &syncList[i].lastSyncTime );
	fm->isChanged( &sm->syncList[i] );

	// 바뀐 파일이 있을 경우에만 동기화 작업을 수행
	// 바뀐 파일이 없을 경우 다음 SyncList로 이동
	if( fm->changeList.empty() )
	{
		return 0;
	}

	// 우선 나 자신을 Lock시키고
	acha->lock();

	// 해당 원격지로 접속
	nRet = cm->connect_client( sm->syncList[i].target, cm->sendPort );
	if( nRet != RC_OK )
	{
		sm->lm.log(LV_INFO, "Remote Connect Fail : %s", sm->syncList[i].target );
		cm->disconnect_client();
		acha->unlock();
		return 0;
	}

	// lock 실행 요청
	nRet = cm->sendCmd( CM_LOCK, sm->m_sCurSyncName );
	if( CM_ACK_OK != nRet )
	{
		sm->lm.log(LV_INFO, "Remote Lock Fail : %s", sm->syncList[i].target );
		acha->unlock();
		return 0;
	}

	// ChangeList를 순회하며 동기화를 수행한다.
	for( int j = 0; j < (int)fm->changeList.size(); j++ )
	{
		lPath = fm->changeList[j].lPath;
		rPath = fm->changeList[j].rPath;

		switch( fm->changeList[j].type )
		{
			case CI_DIRADD :
				bzero( szBuf, sizeof(szBuf) );
				sprintf( szBuf, "%s %d", rPath, getPermission(lPath) );

				nRet = cm->sendCmd(CM_ADD_DIR, szBuf);
				if( CM_ACK_OK != nRet )
				{
				}
				break;

			case CI_FILEADD :
				bzero( md5tmp, sizeof(md5tmp) );
				strcpy( md5tmp, md5file( lPath ) );
				sprintf( szBuf, "%s %s", rPath, md5tmp );

				nRet = cm->sendCmd(CM_ADD_FILE, szBuf);
				if( nRet == CM_ACK_OK )
				{
					// CM을 이용한 파일 전송
					cm->sendFile( lPath, rPath, md5tmp );
					//		이름 변경		cm->sendFile( fm->changeList[j].name,
					//						gen_rpath( syncList[i].localPath,
					//						syncList[i].remotePath,
					//						fm->changeList[j].name ), md5tmp ); 

				}
				else if( nRet == CM_ACK_HASH_EQUAL )
				{
				}
				else
				{
					bUpdateTime	= false;
				}
				break;

			case CI_FILEDEL :
				nRet = cm->sendCmd(CM_DEL_FILE, rPath);
				if( CM_ACK_OK != nRet )
				{
					bUpdateTime	= false;
				}
				break;

			case CI_DIRDEL :
				nRet = cm->sendCmd(CM_DEL_DIR, rPath);
				if( CM_ACK_OK != nRet )
				{
					bUpdateTime	= false;
				}
				break;
		}
		if( !bUpdateTime )
		{
			break;
		}
	}

	if( bUpdateTime )
	{
		nRet = cm->sendCmd(CM_UPDATE_SYNCTIME, syncList[i].syncName);
		if( CM_ACK_OK == nRet )
		{
			updateSyncTime( syncList[i].syncName, time(NULL) );
			fm->writeCFL( syncList[i].syncName );
		}
	}

	// Lock을 해제한다.
	cm->sendCmd( CM_UNLOCK );
	acha->unlock();

	// 접속을 끊는다.
	cm->disconnect_client();

	return 0;
}

void SyncManager::SetCurSyncName( char *sName )
{
	memset( m_sCurSyncName, 0, SDATA );
	snprintf(m_sCurSyncName, SDATA-1, sName );
}
