#include "acha.h"
#include "sm.h"
#include "cm.h"
#include "fm.h"
#include "uim.h"

#ifdef DEBUG
int main( void )
{
	UIModule uim;
	uim.run();

	return 0;
}
#endif

UIModule::UIModule(ACHA *pAcha)
{
	acha = pAcha;
}

UIModule::~UIModule()
{
}

/*
 * 함수 : 초기화 함수
 */
int UIModule::init( void )
{
	//bzero( remoteTarget, sizeof(remoteTarget) );
	bzero( buf, sizeof(buf) );
	bzero( cmdBuffer, sizeof(cmdBuffer) );

	listenPort 	= UIM_PORT;
	sendPort	= UIM_PORT;
	//controlPort	= CONTROL_PORT;

	//isLock 	= false;
        debugMode	= false;
	//demon		= false;
	//debugOn();

	client_len 	= sizeof(client_addr);
        server_len 	= sizeof(server_addr);
	//rtarget_len 	= sizeof(rtarget_addr);

	//bzero( username, sizeof(username) );
	//strncpy( username, getlogin(), sizeof(username) );

	memset( &serverData, 0, sizeof(serverData) );
	memset( &serverResult, 0, sizeof(serverResult) );
	memset( &syncData, 0, sizeof(syncData) );
	memset( &uiappData, 0, sizeof(uiappData) );
	memset( &uiappResult, 0, sizeof(uiappResult) );

	// 디버깅을 위한 타겟 로컬지정
	//strcpy( remoteTarget, "127.0.0.1" );

	lm.SetOutputMode(OUTPUT_FILE);
	lm.SetLogLevel(LV_TRACE);
	lm.SetOwner("UIM");

	return RC_OK;
}

/*
 * 함수 : 실행
 * 설명 : 의미없음. UIM의 기본 실행 루틴
 */
int UIModule::run( void )
{
	if(init_server() != 1)
	{
		lm.log(LV_ERROR, "UIModule::run;init_server()");
		return RC_ERROR;
	}

	bind_server();

	status_uim = pthread_create( &uim_tid, NULL, UIModule::accept_server, acha);

	//status_sm = pthread_create( &sm_tid, NULL, SyncManager::run, this );

	/*
	while( getCmd() )
	{
		receiveDataParsing();
	}
	*/
	return RC_OK;
}

int UIModule::cleanup(void)
{
	return RC_OK;
}

/*
 * 함수 : 사용자로부터 명령을 입력 받음
 * 설명 : UI와 network, 그외 다른 클래스로부터 명령을 입력 받음
 *        디버깅을 위한 함수
 */

bool UIModule::getCmd( void )
{
	if( demon ) pause();

	bzero( cmdBuffer, sizeof(cmdBuffer) );

	setcolor( COLOR_GREEN );
	cout << "UIM::getCmd() > ";
	setcolor( COLOR_NONE );

	fgets( cmdBuffer, DATASIZ, stdin );
	cmdBuffer[strlen(cmdBuffer)-1] = '\0';		// cmdBuffer 에서 '\n' 제거

	if( strncmp( cmdBuffer, "exit", strlen("exit") ) == 0 )
	{
		return false;
	}
	
	return true;
}

/*
 * 함수 : 사용자로부터 명령을 입력 받음
 * 설명 : UI와 network, 그외 다른 클래스로부터 명령을 입력 받음
 *        디버깅을 위한 함수
 */
/*
int UIModule::getCmd( char* cmd )
{
	bzero( cmdBuffer, sizeof(cmdBuffer) );
	strncpy( cmdBuffer, cmd, sizeof(cmdBuffer) );
	
	return receiveDataParsing();RC_ERROR
}
*/
/*
 * 함수 : 받은 데이터 파싱 1
 * 설명 : 받은 데이터에 대해 해당 함수를 호출한다.
 *        전송 받은 데이터는 멤버변수에 저장된다.
 * 반환 : SendCmd 형 플래그
 */

int UIModule::receiveDataParsing( void )
{
	int cmdLength = strlen( cmdBuffer );

	// 엔터 무시
	if( cmdLength == 0 )
	{
		return true;
	}

	return true;
}

/*
 * 함수 : 소켓 통신을 위한 서버 초기화
 * 설명 : 기초적인 서버의 정보를 초기화 한다.
 */
int UIModule::init_server( void )
{
	if( (server_sockfd = socket(PF_INET, SOCK_STREAM, 0)) < 0 )
	{
		perror("init_server() socket error : ");
		return 0;
	}
 
	bzero( &server_addr, sizeof(server_addr) );
        server_addr.sin_family = AF_INET;
        server_addr.sin_addr.s_addr = htonl(INADDR_ANY);		// 자신의 IP 사용
        server_addr.sin_port = htons( listenPort );

	return 1;
}

/*Acha *acha = (Acha *)pthis;
	UIModule *uim = acha->uim;
 * 함수 : 서버 소켓의 주소 할당
 * 설명 : 기본 포트 이후로 검색하며 비어 있는 포트에 주소를 할당
 *        할당된 포트는 멤버변수 listenPort에 저장
 */
void UIModule::bind_server( void )
{
    int state = -1;

	server_addr.sin_port = htons( listenPort );
	state = bind( server_sockfd, (struct sockaddr *)&server_addr, server_len );

	// 포트 할당 성공
	if( state >= 0 )
	{
		cout << "[1;36m";
		cout << "포트번호 : " << listenPort << endl;
		cout << "[00m";
	}

	/*
	// 포트 할당
	while( state < 0 )
	{
		server_addr.sin_port = htons( listenPort );
		state = bind( server_sockfd, (struct sockaddr *)&server_addr, server_len );

		if( state < 0 )
		{
			if( errno == EADDRINUSE )
			{
				setcolor( COLOR_RED );
				cout << listenPort << " 포트 사용중" << endl;
				cout << "새로운 포트 할당 시도중.." << endl;
				setcolor( COLOR_NONE );

				listenPort++;

				if( listenPort > UIM_PORT_END )
				{
					cout << "UIM::bind_server() - 포트 할당 실패." << endl;
					exit(0);
				}

				continue;
			}
			lm.log(LV_ERROR, "bind_server() bind error");
		}

		// 포트 할당 성공
		if( state >= 0 )
		{
			cout << "[1;36m";
			cout << "포트번호 : " << listenPort << endl;
			cout << "[00m";
			break;
		}
	}
	*/
	state = listen( server_sockfd, 10 );

	if( state == -1 )
	{
		perror("bind_server() listen error : ");
		exit(0);
	}
	
	return;
}

/*
 * 함수 : buf 에 있는 내용을 원격지로 전송한다.
 * 인자 : 없음
 * 반환 : 성공 : 0, 실패 : -1
 */
/*
int UIModule::sendCmd( void )
{
	if( rtarget_sockfd <= 0 )
	{
		if( debugMode ) printf("rserver_sockfd is not valid.\n");
		sendResult = UIM_ACKNO;
		return UIM_ACKNO;
	}

	// 전송을 정해진 양만큼만 보냄
	write( rtarget_sockfd, &uiappData, 
		   sizeof(uiappData.type) + sizeof(uiappData.len) + strlen(uiappData.data) );
	if( debugMode )
	{
		setcolor( COLOR_GREEN );
		printf("[Send] [%d] [%d] [%s] -- ", 
				uiappData.type, uiappData.len, uiappData.data  );
		setcolor( COLOR_NONE );
	}

	// 결과값 수신
	memset( &uiappResult, 0, sizeof( uiappData ) );
	read( rtarget_sockfd, &uiappResult, sizeof(uiappResult.type) + sizeof(uiappResult.len) );
	if( uiappResult.len > 0 )
	{
		read( rtarget_sockfd, &uiappResult.data, uiappResult.len );
	}

	if( debugMode )
	{
		setcolor( COLOR_SKY );
		printf("[Receive] [%d] [%d] [%s]\n", 
				uiappResult.type, uiappResult.len, uiappResult.data  );
		setcolor( COLOR_NONE );
	}

	return uiappResult.type;
}
*/

/*
 * 함수 : 원격지와의 연결을 종료한다.
 * 인자 : 없음
 * 반환 : 성공 : 0, 실패 : 1
 */
int UIModule::disconnect_client( void )
{
	close( rtarget_sockfd );

	return 0;
}

/*
 * Thread Function ===================================
 * 함수 : 타이머 구현 함수
 * 설명 : 주기적으로 실행할 함수를 등록한다.
 */
/*
void *UIModule::timer( void *pthis )
{
	UIModule *uim = (UIModule *)pthis;

	while( 1 )
	{
		sleep( 1 );

		if( uim->isLock )
		{
			continue;
		}

		//uim->portRefresh();
	}

	pthread_exit(NULL);
	return 0;
}
*/
/*
 * Thread Function ===================================
 * 함수 : 클라이언트의 접속을 처리한다.
 * 설명 : 서버의 기능으로 접속을 기다리며
 * 		  새로운 클라이언트가 접속하면 스레드를 생성하여
 * 		  명령을 처리한다.
 */
void *UIModule::accept_server( void *pthis )
{
	ACHA *acha = (ACHA *)pthis;
	UIModule *uim = (UIModule *)acha->m_pUIModule;

	// 다음 접속을 기다린다.완료
	while( 1 )
	{		
		uim->client_sockfd = accept( uim->server_sockfd, 
				(struct sockaddr *)&uim->server_addr, 
				&uim->server_len );
		if( uim->client_sockfd == -1 )
		{
			//lm.log(LV_ERROR, "clientConnect() Accept error");
			perror("clientConnect() Accept error : ");
			return 0;
		}

		uim->status_uim = pthread_create( &uim->uim_tid, NULL, UIModule::clientConnect,	acha );
	}

	return 0;
}

/*
 * 함수 : 동기화 정보를 UIAPP로 송출한다.
 */
int UIModule::view_SyncInfo( void *pthis )
{
	ACHA *acha = (ACHA *)pthis;
	UIModule *uim = (UIModule *)acha->m_pUIModule;
	SyncManager *smp = (SyncManager *)acha->m_pSyncManager;
	CommunicationManager *cmp = (CommunicationManager *)acha->m_pCommunicationManager;
	//FileManager *fmp = (FileManager *)acha->m_pFileManager;

	int clientSockFd = uim->client_sockfd;	

	//uim->syncData.type = 1;
	//sprintf( uim->syncData.data, "%s|%s|%s|%s", rPath, totalSize, hashcode, filePermission );	
	//uim->syncData.len = strlen(uiappData.data);

	//write( clientSockFd, "123", strlen("123"));
	//write( clientSockFd, "123123", strlen("123123"));
	//write( clientSockFd, "123123123", strlen("123123123"));

	memset( &syncData, 0, sizeof(syncData) );
	uim->syncData.type = 1;
		
	sprintf( uim->syncData.data, "%s|%s|%s|%s|%s|%d", (acha->m_bLock? "true" : "false"), (acha->m_bDebug ? "true" : "false"), (strlen(acha->username) <= 0 ? " " : acha->username), (strlen(acha->userpass) <= 0 ? " " : acha->userpass), acha->recordLockPath, acha->m_fdRecordLock );			
	uim->syncData.len = strlen(uim->syncData.data);
	write( clientSockFd, &uim->syncData, 
	   sizeof(uim->syncData.type) + sizeof(uim->syncData.len) + strlen(uim->syncData.data) );

	printf("UIModule::view_SyncInfo; smp->syncList.size(); %d\n", (int)smp->syncList.size());
	
	for(int i = 0; i < (int)smp->syncList.size(); i++)
	{
		memset( &syncData, 0, sizeof(syncData) );
		uim->syncData.type = (int)smp->syncList.size();
		
		sprintf( uim->syncData.data, "%s|%s|%s|%s|%s", smp->syncList[i].syncName, smp->syncList[i].target, smp->syncList[i].localPath, smp->syncList[i].remotePath, timeFormat(smp->syncList[i].lastSyncTime));
				
		uim->syncData.len = strlen(uim->syncData.data);

		printf("uim->syncData.type : %d \n", uim->syncData.type);
		printf("uim->syncData.data : %s \n", uim->syncData.data);
		printf("uim->syncData.len : %d \n", uim->syncData.len);

		write( clientSockFd, &uim->syncData, 
		   sizeof(uim->syncData.type) + sizeof(uim->syncData.len) + strlen(uim->syncData.data) );
		
	}	

	if((int)smp->syncList.size() > 0)
	{
		memset( &syncData, 0, sizeof(syncData) );
		uim->syncData.type = (int)smp->syncList.size();

		//sprintf( uim->syncData.data, "%d|%d|%d|%d|%s|%d|%d|%s|%d|%d|%s|%s|%d|%d|%s", cmp->isLock, cmp->debugMode, cmp->server_sockfd, cmp->listenPort, cmp->client_addr.sin_addr, cmp->serverData.type, cmp->serverData.len, cmp->serverData.data, cmp->rtarget_sockfd, cmp->sendPort, cmp->remoteTarget, cmp->rtarget_addr.sin_addr, cmp->clientData.type, cmp->clientData.len, cmp->clientData.data);
		//sprintf( uim->syncData.data, "%d|%d|%d|%d|%s|%d|%d|%s|%d|%d|%s|%s|%d|%d|%s", acha->m_bLock, acha->m_bDebug, cmp->server_sockfd, cmp->listenPort, inet_ntoa(cmp->client_addr.sin_addr), cmp->serverData.type, cmp->serverData.len, cmp->serverData.data, cmp->rtarget_sockfd, cmp->sendPort, cmp->remoteTarget, inet_ntoa(cmp->rtarget_addr.sin_addr), cmp->clientData.type, cmp->clientData.len, cmp->clientData.data);

		sprintf( uim->syncData.data, "%d|%d|%s|%d|%d|%s|%d|%d|%s|%s|%d|%d|%s", cmp->server_sockfd, cmp->listenPort, inet_ntoa(cmp->client_addr.sin_addr), cmp->serverData.type, cmp->serverData.len, cmp->serverData.data, cmp->rtarget_sockfd, cmp->sendPort, cmp->remoteTarget, inet_ntoa(cmp->rtarget_addr.sin_addr), cmp->clientData.type, cmp->clientData.len, cmp->clientData.data);
		
		uim->syncData.len = strlen(uim->syncData.data);

		write( clientSockFd, &uim->syncData, 
		   sizeof(uim->syncData.type) + sizeof(uim->syncData.len) + strlen(uim->syncData.data) );
	}

	if((int)smp->syncList.size() <= 0)
	{
		memset( &syncData, 0, sizeof(syncData) );
		uim->syncData.type = 0;		
		sprintf( uim->syncData.data, "%s", "");				
		uim->syncData.len = strlen(uim->syncData.data);
		write( clientSockFd, &uim->syncData, 
		   sizeof(uim->syncData.type) + sizeof(uim->syncData.len) + strlen(uim->syncData.data) );

		uim->syncData.type = 0;		
		sprintf( uim->syncData.data, "%s", "");				
		uim->syncData.len = strlen(uim->syncData.data);
		write( clientSockFd, &uim->syncData, 
		   sizeof(uim->syncData.type) + sizeof(uim->syncData.len) + strlen(uim->syncData.data) );
	}

	/*
	if((int)fmp->scanList.size() <= 0)
	{
		uim->syncData.type = 0;		
		sprintf( uim->syncData.data, "%s", "");				
		uim->syncData.len = strlen(uim->syncData.data);
		write( clientSockFd, &uim->syncData, 
		   sizeof(uim->syncData.type) + sizeof(uim->syncData.len) + strlen(uim->syncData.data) );
	}	
	else
	{
		for(unsigned int i = 0; i < fmp->scanList.size(); i++)
		{	
			memset( &syncData, 0, sizeof(syncData) );	
			uim->syncData.type = (int)fmp->scanList.size();
		
			sprintf( uim->syncData.data, "%d|%s|%s", fmp->scanList[i].type, fmp->scanList[i].name, timeFormat(fmp->scanList[i].mod_time));
				
			uim->syncData.len = strlen(uim->syncData.data);

			printf("%d\n", fmp->scanList[i].type);
			printf("%s\n", fmp->scanList[i].name);
			printf("%s\n",timeFormat(fmp->scanList[i].mod_time));


			write( clientSockFd, &uim->syncData, 
			   sizeof(uim->syncData.type) + sizeof(uim->syncData.len) + strlen(uim->syncData.data) );
		}
	}
	

	if((int)fmp->lastList.size() <= 0)
	{
		uim->syncData.type = 0;		
		sprintf( uim->syncData.data, "%s", "");				
		uim->syncData.len = strlen(uim->syncData.data);
		write( clientSockFd, &uim->syncData, 
		   sizeof(uim->syncData.type) + sizeof(uim->syncData.len) + strlen(uim->syncData.data) );
	}
	else
	{
		for(unsigned int i = 0; i < fmp->lastList.size(); i++)
		{	
			memset( &syncData, 0, sizeof(syncData) );	
			uim->syncData.type = (int)fmp->lastList.size();
		
			sprintf( uim->syncData.data, "%d|%s|%s", fmp->lastList[i].type, fmp->lastList[i].name, timeFormat(fmp->lastList[i].mod_time));
				
			uim->syncData.len = strlen(uim->syncData.data);

			printf("%d\n", fmp->lastList[i].type);
			printf("%s\n", fmp->lastList[i].name);
			printf("%s\n", timeFormat(fmp->lastList[i].mod_time));

			write( clientSockFd, &uim->syncData, 
			   sizeof(uim->syncData.type) + sizeof(uim->syncData.len) + strlen(uim->syncData.data) );	
		
		}
	}

	if((int)fmp->changeList.size() <= 0)
	{
		uim->syncData.type = 0;		
		sprintf( uim->syncData.data, "%s", "");				
		uim->syncData.len = strlen(uim->syncData.data);
		write( clientSockFd, &uim->syncData, 
		   sizeof(uim->syncData.type) + sizeof(uim->syncData.len) + strlen(uim->syncData.data) );
	}
	else
	{
		for(unsigned int i = 0; i < fmp->changeList.size(); i++)
		{
			memset( &syncData, 0, sizeof(syncData) );	
			uim->syncData.type = (int)fmp->changeList.size();
		
			sprintf( uim->syncData.data, "%d|%s|%s", fmp->changeList[i].type, fmp->changeList[i].lPath, fmp->changeList[i].rPath);
				
			uim->syncData.len = strlen(uim->syncData.data);

			printf("%d\n", fmp->changeList[i].type );
			printf("%s\n", fmp->changeList[i].lPath);
			printf("%s\n", fmp->changeList[i].rPath);

			write( clientSockFd, &uim->syncData, 
			   sizeof(uim->syncData.type) + sizeof(uim->syncData.len) + strlen(uim->syncData.data) );			
		}	
	}
	*/
	
	printf("\nUIModule::view_SyncInfo; 실행 결과 송출 완료!!!\n");

	return 0;

}

/*
 * Thread Function ===================================
 * 함수 : 클라이언트의 접속을 처리한다.
 * 설명 : 서버의 기능으로 접속을 기다리며
 * 		  접속된 클라이언트로부터 명령을 받아 처리한다
 * 		  더이상 받을 메시지가 없으면 종료
 */
void *UIModule::clientConnect( void *pthis )
{
	ACHA *acha = (ACHA *)pthis;
	UIModule *uim = (UIModule *)acha->m_pUIModule;
	SyncManager				*smp 	= (SyncManager *)acha->m_pSyncManager;
	CommunicationManager 	*cmp 	= (CommunicationManager *)acha->m_pCommunicationManager;

	int clientSockFd = uim->client_sockfd;
	clientInfo cltmp;

	// 스레드의 TID를 얻어오는 함수 찾아서 집어 넣을 것.
	cltmp.tid = uim->uim_tid;
	cltmp.clientSockFd = clientSockFd;
	cltmp.clientIP = inet_ntoa( uim->client_addr.sin_addr );

	// 접속중인 클라이언트 리스트에 삽입
//	uim->clientList.push_back( cltmp );

	if( uim->debugMode )
	{
		setcolor( COLOR_YELLOW );
		cout << "\nClient Connected::sockfd = " << clientSockFd;
		cout << " (form " << cltmp.clientIP << ")" << endl;
		setcolor( COLOR_NONE );
	}

	int len;
	// 명령을 읽어온다
	while( 1 )
	{
		memset( &uim->uiappData, 0, sizeof(uim->uiappData) );
		len = read( clientSockFd, &uim->uiappData,
				sizeof(uim->uiappData.type) + sizeof(uim->uiappData.len) );
		if( uim->uiappData.len > 0 )
		{
			read( clientSockFd, &uim->uiappData.data, uim->uiappData.len );
		}

		if( (uim->uiappData.type == CM_LOCK) && uim->isLock )
		{
			write( clientSockFd, 
					uim->makeCmd(CM_ACK_NO, "locking"),
					4 + 4 + strlen("locking..") );
		}

		if( len <= 0 )
		{
			close( clientSockFd );

			if( uim->debugMode )
			{
				setcolor( COLOR_YELLOW );
				cout << "\n(" << len << ")DisConnected::sockfd = " << clientSockFd << endl;
				setcolor( COLOR_NONE );
			}

			break;
		}
		
		//////////////////////////////////////////////////
		// 읽어온 명령을 해석한다
		////////////////////////////////////////////////// 

		char in_data[256];
		char name[SDATA]  	= "";
		char target[SDATA] 	= "";
		char lpath[SDATA]  	= "";
		char rpath[SDATA]  	= "";
		char id[SDATA]     	= "id";
		char pass[SDATA] 	= "pass";
		int  filesize  		= 10000000;
		int  flag		= FLAG_NEWSYNC;
		int chk_ok = 0;

		sprintf(in_data, "%s", uim->uiappData.data);

		if( uim->debugMode )
		{
			cout << "(" << clientSockFd << ")reuim->uiappData.typeceive :: " << uim->uiappData.data << endl;
		}

		switch( uim->uiappData.type )
		{
			case UIM_CREATE_SYNC:	
				if( argc( in_data ) == 4 )
				{
					sscanf( in_data, "%s %s %s %s", name, target, lpath, rpath );
					printf("name1 : %s\n", name);
					printf("target1 : %s\n", target);
					printf("lpath1 : %s\n", lpath);
					printf("rpath1 : %s\n", rpath);					
				}
				else 
				{
					sscanf( in_data, "%s %s %s %s %d", name, target, lpath, rpath, &flag );
					printf("name2 : %s\n", name);
					printf("target2 : %s\n", target);
					printf("lpath2 : %s\n", lpath);
					printf("rpath2 : %s\n", rpath);									
				}

				// 원격지 주소가 IP일 경우
				if( isIPformat( target ) )
				{
					uim->rtarget_addr.sin_addr.s_addr = inet_addr( target );
				}
				// 원격지 주소가 domain name 일 경우
				else
				{
					struct hostent *he;
					he = gethostbyname( target );

					// gethostbyname 결과 IP를 받아오지 못했을 경우
					if( he == NULL )
					{
						cmp->lm.log(LV_ERROR, "gethostbyname 결과 IP를 받아오지 못했을 경우");
					}
					else
					{
						uim->rtarget_addr.sin_addr.s_addr = inet_addr( inet_ntoa( *(struct in_addr*)he->h_addr) );
					}
				}

				chk_ok = smp->addSync( name, target, lpath, rpath, flag, id, pass , filesize );

				sprintf(uim->uiappData.data, "%d", chk_ok);

				printf("uim->uiappData.data = %s\n", uim->uiappData.data);

				// 실행 결과 송출		
				write( clientSockFd, uim->uiappData.data, sizeof(uim->uiappData.data));			
			
				break;

			case UIM_DELETE_SYNC:	
				chk_ok = smp->delSync( in_data );	
				sprintf(uim->uiappData.data, "%d", chk_ok);
				// 실행 결과 송출		
				write( clientSockFd, uim->uiappData.data, sizeof(uim->uiappData.data));	
				break;

			case UIM_VIEW_SYNCINFO:
				printf("uim->uiappData.type :UIM_VIEW_SYNCINFO\n");	
				uim->view_SyncInfo(acha);
				break;
		}


		printf("\n*UIModule::clientConnect; 실행 결과 송출 완료\n");
	}


	return 0;
}

/*	
 * 함수 : 커맨드를 만들어 리턴한다.
 * 설명 : 스태틱 변수를 사용하므로 일회성 커맨드임을 명심!!
 */
char *UIModule::makeCmd( int type )
{
	return makeCmd( type, NULL );
}
char *UIModule::makeCmd( int type, char *data )
{
	static char buf[sizeof(cmdData)];
	int len = strlen(data);
	strncpy( buf, data, len );
	memcpy( &buf[0], &type, sizeof(int) );
	memcpy( &buf[4], &len, sizeof(int) );
	memcpy( &buf[8], data, len );
	return buf;
}




