#ifndef __UIMODULE_H__
#define __UIMODULE_H__

#include "acha.h"

// Communication 메인 클래스
class UIModule : public ACHABase
{
	private :
		char username[SDATA];
		char userpass[SDATA];
		char recordLockPath[SDATA];
		int fdRecordLock;

		// 스레드 관련변수
		void *threadResult;			// thread 반환값 참조

		int status_timer;			// portRefresh Timer
		pthread_t	timer_tid;		// portRefresh Timer

		int status_uim;				// communication server
		pthread_t	uim_tid;			// accept 스레드의 tid

		int status_sm;				// communication server
		pthread_t	sm_tid;			// accept 스레드의 tid

		// 소켓 관련변수
		int listenPort;
		int sendPort;
		int controlPort;

		int control_sockfd;				// 컨트롤 서버 소켓
		int server_sockfd;				// 서버 소켓
		int rtarget_sockfd;				// 원격지 서버 소켓
		int client_sockfd;				// 클라이언트 소켓

		struct sockaddr_in control_addr;// 컨트롤 서버 주소
		struct sockaddr_in server_addr;	// 서버 주소
		struct sockaddr_in client_addr;	// 클라이언트 주소
		struct sockaddr_in rtarget_addr;// 원격지 주소

		socklen_t control_len;				// 컨트롤 서버 소켓 크기
		socklen_t server_len;				// 서버 소켓 크기
		socklen_t client_len;				// 클라이언트 소켓 크기
		socklen_t rtarget_len;				// 원격지 소켓 크기

		cmdData serverData;				// 서버 데이터 구조체
		cmdData serverResult;				// 서버 데이터 구조체
		cmdData syncData;				
		cmdData uiappData;				// uiapp 데이터 구조체
		cmdData uiappResult;			 	// uiapp 데이터 구조체

		int receiveResult;				// 데이터 수신 후 결과
		int sendResult;					// 데이터 전송 후 결과
		char buf[DATASIZ];				// 버퍼
		char cmdBuffer[DATASIZ];			// 입력버퍼
		char md5buf[SDATA];				// md5 해쉬코드 임시저장소

		struct termios termSettings;	// 터미널 세팅

	public :
		UIModule(ACHA*);
		~UIModule();

		ACHA            *acha;

		bool isLock;					// 실행정지
		bool debugMode;					// 디버그 모드
		bool demon;						// 데몬 모드

		time_t lockTime;
		char remoteTarget[SDATA];		// 접속할 원격지

		list<clientInfo> clientList;
		//SyncManager 	*sm;
		//FileManager 	*fm;

		int init( void );				// 초기화
		int cleanup( void );			// 정리
		int run( void );				// CM 실행

		int view_SyncInfo( void *);

		int init_server( void );				// 소켓 초기화
		void bind_server( void );				// 포트할당
		static void *accept_server( void * );	// 접속
		static void *clientConnect( void * );	// 접속 처리
		void close_server( void );				// 소켓 종료
		//void portRefresh( void );				// 상위포트로이동

		//void init_control_server( void );		// 컨트롤 서버 실행
		//void close_control_server( void );		// 컨트롤 서버 실행 중지

		bool getCmd( void );				// UI로부터 명령을 입력 받음
		int  getCmd( char* );				// 프로그램으로 부터 명령을 받음

		int receiveDataParsing( void );	// 받은 명령어 분석

		int init_client( void );			// client를 초기화 한다.
		int connect_client( void );			// 원격지에 접속한다.
		int disconnect_client( void );		// 원격지와의 접속을 종료한다.
		int sendCmd( void );				// 명령을 전송한다
		int reply( void );					// 현재 접속중인 클라이언트에 대답한다.
		char *makeCmd( int );				// 전송단위 명령을 만든다
		char *makeCmd( int, char* );		// 전송단위 명령을 만든다

		//bool invalidTarget( char * );		// 유효하지 않은 대상인지 확인
		//void debug( void );					// 현재 설정을 모두 출력함

		//void echoOFF( void );
		//void echoON( void );

		//int addSync( char * );
		//int delSync( char * );				// 동기화 규칙을 지운다.

		//void showList( void );				// 싱크 리스트를 보여준다.

		/////////////////////////////////
		// Sync Manager Support Function
		char *getPassword( void );			// 프로그램이 실행 될 때 저장된 
											// 패스워드를 알려준다
		void setPassword( void );			// 프로그램의 패스워드를 설정

		/////////////////////////////////
		// Transfer Manager Function
		int totalSize;
		int sendAmount;
		int receiveAmount;

		FILE *fpTemp;

		int sendFile( char *, char *, char * );
		int receiveFile( void );

		// 인라인 함수
		void lock( void )
		{
			isLock = true;
			lockTime = time(NULL);
		};

		void unlock( void )
		{ 
			isLock = false; 
		};

		void debugOn( void ) 
		{ 
			debugMode = true; 
		};

		void debugOff( void )
		{ 
			debugMode = false; 
		};
};
#endif
